/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	GESHDR:					*/
/*	Take Care of all E-Mail neader analyse	*/
/*						*/
/************************************************/
%{
/*force le debugging du YACC                    */
#define lint    1	/*To avoid yacc msg	*/

#include	<string.h>
#include	<setjmp.h>
#include	<syslog.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	"geshdr.h"
#include	"subrou.h"

int yyparse();		/*syntax parser		*/
static int yylex();	/*lexical parser	*/

/*preamble static function declaration		*/
static int skipfor(char);
static void yyerror(char *);
static char *strmix(char *,char,char *);




typedef	enum	{	/*header type frame	*/
	hd_from,	/*From:			*/
	hd_to,		/*To:			*/
	hd_cc,		/*Cc:			*/
	hd_bcc,		/*Bcc:			*/
	hd_sender,	/*Sender: 		*/
	hd_replyto,	/*Reply-To: 		*/
	hd_inreplyto,	/*In-Reply-To: 		*/
	hd_references,	/*References: 		*/
	hd_msgid,	/*Message-ID:		*/
	hd_subject,	/*Subject:		*/
	hd_date		/*Date:			*/
	}HIDTYP;

typedef	struct	{
	HIDTYP field;	/*header field type	*/	
	union {
	  char *time;	/*header date		*/
	  char *address;/*From address		*/
	  char *subject;/*Subject		*/
	  char **maddr;	/*To address		*/
	  }info;
	}HDRTYP;

static HDRTYP **ghdrs;	/*header collected info	*/
static int itsok;	/*parsing status	*/
static jmp_buf manystop;/*YACC fully lost	*/
static int cptstop=0;	/*stop received counting*/

static char **lines;	/*data-header pointer	*/
static int numline;	/*next char header line	*/
static int numrow;	/*next char header row	*/
static int curline;	/*cur char header line	*/
static int currow;	/*cur char header row	*/
static int nbrerr;	/*nombre d'erreur	*/


/*

*/
/************************************************/
/*						*/
/*	Procedure to create the memory space	*/
/*	for a header line			*/
/*						*/
/************************************************/
static HDRTYP *genheader(HIDTYP field,void *infos)

{
HDRTYP *header;

header=(HDRTYP *)calloc(1,sizeof(HDRTYP));
header->field=field;
switch (field) {
  case (hd_from)	:
  case (hd_sender)	:
  case (hd_replyto)	:
  case (hd_msgid)	:
  case (hd_date)	:
    header->info.address=(char *)infos;
    break;
  case (hd_to)		:
  case (hd_cc)		:
  case (hd_bcc)		:
  case (hd_references)	:
  case (hd_inreplyto)	:
    header->info.maddr=(char **)infos;
    break;
  case (hd_subject)	:
    header->info.subject=(char *)infos;
    break;
  default		:
    (void) tracing(TRALOG,0,"geshdr.y,genheader: Unexpected field (bug?)");
    break;
  }
return header;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add a header component to	*/
/*	the header list.			*/
/*						*/
/************************************************/
static HDRTYP **addhdrs(HDRTYP **chdrs,HDRTYP *header)

{
if (header!=(HDRTYP *)0) {
  chdrs=(HDRTYP **)rou_addveclst((void **)chdrs,(void *)header);
  }
return chdrs;
}
%}

%union  {
	HDRTYP **hdrs;	/*full header parsed	*/
	HDRTYP *header;	/*one line header	*/
	char *string;	/*the info is a string	*/
	char punct;	/*one char only		*/
	char *word;	/*the info is a word	*/
	char *email;	/*the info is an E-mail	*/
	char **mboxes;	/*multiple Mail boxes	*/
	void *generic;	/*generic value		*/
	long time;	/*time value as a long	*/
	char *strtime;	/*time as a str(long)	*/
	int symbol;	/*token numeric value	*/
	long integer;	/*the value is integer	*/
	float floatting;/*the value is a float	*/
	};

%token	STOP	/*Header part end		*/
%token	CRLF	/*Carriage return		*/
%token	COLON	/* ':' character		*/
%token	SEMICOL	/* ';' character		*/
%token	COMMA	/* ',' character		*/
%token	AT	/* '@' character		*/
%token	DOT	/* '.' character		*/
%token	MINUS	/* '-' character		*/
%token	PLUS	/* '+' character		*/
%token	PARLEFT	/* '(' character		*/
%token	PARRIGHT/* ')' character		*/
%token	BRKLEFT	/* '[' character		*/
%token	BRKRIGHT/* ']' character		*/
%token	LESSTHAN/* '<' character		*/
%token	GRTRTHAN/* '>' character		*/
%token	QUESTION/* '?' character		*/
%token	DEBENCOD/* '=?' characters		*/
%token	ENCODFIN/* '?=' characters		*/
%token	PUNCT	/* any other non alphanum char	*/

%token	DATE	/* "Date"			*/
%token	RDATE	/* "Resent-Date"		*/
%token	FROM	/* "From"			*/
%token	SUBJECT	/* "Subject"			*/
%token	REPLYTO	/* "Reply-To"			*/
%token	INRPLYTO/* "In-Reply-To"		*/
%token	RFERENCE/* "References"			*/
%token	SENDER	/* "Sender"			*/
%token	MSGID	/* "Message-ID"			*/
%token	TO	/* "To"				*/
%token	CC	/* "Cc"				*/
%token	BCC	/* "Bcc"			*/

%token	WORD	/*a word with letter and numbers*/
%token	STRING	/*litteral charactere		*/

/*Month name					*/
%token	JAN	/* "Jan"			*/
%token	FEB	/* "Feb"			*/
%token	MAR	/* "Mar"			*/
%token	APR	/* "Apr"			*/
%token	MAY	/* "May"			*/
%token	JUN	/* "Jun"			*/
%token	JUL	/* "Jul"			*/
%token	AUG	/* "Aug"			*/
%token	SEP	/* "Sep"			*/
%token	OCT	/* "Oct"			*/
%token	NOV	/* "Nov"			*/
%token	DEC	/* "Dec"			*/

/*Day name					*/
%token	MON	/* "Mon"			*/
%token	TUE	/* "Tue"			*/
%token	WED	/* "Wed"			*/
%token	THU	/* "Thu"			*/
%token	FRI	/* "Fri"			*/
%token	SAT	/* "Sat"			*/
%token	SUN	/* "Sun"			*/
/*
^L
*/
/************************************************/
/*	E-Mail header definition		*/
/************************************************/
%start          email

%%
email		:	headers STOP
			{
			ghdrs=$<hdrs>1;
                        if (itsok==true)
                          YYACCEPT;
                        else
                          YYABORT;
			}
		|	error STOP
			{
			YYABORT;
			}
		;

headers		:	header
			{
			$<hdrs>$=addhdrs((HDRTYP **)0,$<header>1);
			ghdrs=$<hdrs>$;
			}
		|	headers header
			{
			$<hdrs>$=addhdrs($<hdrs>1,$<header>2);
			ghdrs=$<hdrs>$;
			}
		;

header		:	data CRLF
			{
			$<header>$=$<header>1;
			}
		;

data		:	originator
			{
			$<header>$=$<header>1;
			}
		|	destination
			{
			$<header>$=$<header>1;
			}
		|	dates
			{
			$<header>$=$<header>1;
			}
		|	optionalfield
			{
			$<header>$=$<header>1;
			}
		|	WORD
			{
			(void) free($<word>1);
			$<header>$=(HDRTYP *)0;
			if (numrow>0)
			  while (lines[numline][numrow]!='\000')
			     numrow++;
			}
		|	scrap
			{
			$<header>$=(HDRTYP *)0;
			if (numrow>0)
			  while (lines[numline][numrow]!='\000')
			     numrow++;
			(void) tracing(TRALOG,2,"header error!");
			}
		;

scrap		:	others
			{
			if ($<word>1!=(char *)0)
			  (void) free($<word>1);
			}
		|	LESSTHAN
		|	GRTRTHAN
		|	error
		;
/*

*/
/************************************************/
/*	Originator E-Mail header definition	*/
/************************************************/
originator	:	senders mailbox
			{
			$<header>$=(HDRTYP *)0;
			if ($<email>2!=(char *)0)
			  $<header>$=genheader($<integer>1,$<generic>2);
			}
		;

senders		:	FROM COLON
			{
			$<integer>$=(int)hd_from;
			}
		|	SENDER COLON
			{
			$<integer>$=(int)hd_sender;
			}
		|	REPLYTO COLON
			{
			$<integer>$=(int)hd_replyto;
			}
		;

destination	:	rcpts address
			{
			$<header>$=(HDRTYP *)0;
			if ($<mboxes>2!=(char **)0)
			  $<header>$=genheader($<integer>1,$<generic>2);
			}
		|	bcc address
			{
			$<header>$=(HDRTYP *)0;
			if ($<mboxes>2!=(char **)0)
			  $<header>$=genheader($<integer>1,$<generic>2);
			}
		;

rcpts		:	TO COLON
			{
			$<integer>$=(int)hd_to;
			}
		|	CC COLON
			{
			$<integer>$=(int)hd_cc;
			}
		;

bcc		:	BCC COLON
			{
			$<integer>$=(int)hd_bcc;
			}
		;

optionalfield	:	hmsgid msgid
			{
			$<header>$=(HDRTYP *)0;
			if ($<mboxes>2!=(char **)0)
			  $<header>$=genheader($<integer>1,$<generic>2);
			}
		|	inrplyto mulids
			{
			$<header>$=(HDRTYP *)0;
			if ($<mboxes>2!=(char **)0)
			  $<header>$=genheader($<integer>1,$<generic>2);
			}
		|	references mulids
			{
			$<header>$=(HDRTYP *)0;
			if ($<mboxes>2!=(char **)0)
			  $<header>$=genheader($<integer>1,$<generic>2);
			}
		|	SUBJECT COLON
			{
			#define MXSUB	120

			int i;
			char *ptr;

			ptr=strdup(&(lines[numline][numrow]));
			if (strlen(ptr)>MXSUB)
			  ptr[MXSUB]='\000';
			for (i=0;ptr[i]!='\000';i++)
			  if (ptr[i]<' ')
			    ptr[i]=' ';
			while (ptr[0]==' ')
			  (void) strcpy(ptr,ptr+1);
			$<header>$=genheader((int)hd_subject,(void *)ptr);
			if (lines[numline]!=(char *)0) {
			  while (lines[numline][numrow]!='\000')
			     numrow++;
			  }
			}
		;


hmsgid		:	MSGID COLON
			{
			$<integer>$=(int)hd_msgid;
			}
		|	MSGID error
			{
			(void) yyerror("Expecting ':'");
			itsok=false;	/*Forced fatal Error (debugging purpose)	*/
			}
		;

inrplyto	:	INRPLYTO COLON
			{
			$<integer>$=(int)hd_inreplyto;
			}
		;

references	:	RFERENCE COLON
			{
			$<integer>$=(int)hd_references;
			}
		;

mulids		:	msgids
			{
			$<mboxes>$=$<mboxes>1;
			}
		|	
			{
			$<mboxes>$=(char **)0;
			}
		;

msgids		:	msgid
			{
			$<mboxes>$=(char **)0;
			if ($<email>1!=(char *)0)
  			  $<mboxes>$=(char **)rou_addveclst((void **)0,
						           (void *)$<email>1);
			}
		|	msgids msgid
			{
			$<mboxes>$=$<mboxes>1;
			if ($<email>2!=(char *)0)
  			  $<mboxes>$=(char **)rou_addveclst((void **)$<mboxes>1,
						           (void *)$<email>2);
			}
		;

msgid		:	LESSTHAN addrspec GRTRTHAN
			{
			$<email>$=$<email>2;
			(void) rou_strtolower($<email>$); 
			(void) skipfor('<');
			}
		|	LESSTHAN localpart GRTRTHAN
			{
			if ($<email>2!=(char *)0) {
			  (void) free($<email>2);
			  $<email>2=(char *)0;
			  }
			$<email>$=$<email>2;
			(void) rou_strtolower($<email>$); 
			(void) skipfor('<');
			}
		|	LESSTHAN GRTRTHAN
			{
			$<email>$=(char *)0;
			(void) skipfor('<');
			}
		|	WORD
			{
			if ($<word>1!=(char *)0)
			  (void) free($<word>1);
			$<email>$=(char *)0;
			(void) skipfor('<');
			}
		|	others
			{
			if ($<word>1!=(char *)0)
			  (void) free($<word>1);
			$<email>$=(char *)0;
			(void) skipfor('<');
			}
		;

/*

*/
/************************************************/
/*	Date E-Mail header definition		*/
/************************************************/
dates		:	origdate
			{
			$<header>$=$<header>1;
			}
		|	resentdate
			{
			$<header>$=$<header>1;
			}
		;

origdate	:	DATE COLON datetime
			{
			$<header>$=genheader(hd_date,$<strtime>3);
			}
		;


resentdate	:	RDATE COLON datetime
			{
			$<header>$=genheader(hd_date,$<strtime>3);
			}
		;

datetime	:	days COMMA cachet
			{
			$<strtime>$=$<strtime>3;
			}
		|	cachet
			{
			$<strtime>$=$<strtime>1;
			}
		;

cachet		:	date time
			{
			$<strtime>$=(char *)calloc(1,20);	
			(void) sprintf($<strtime>$,"%ld",$<integer>1+$<integer>2);
			}
		|	date
			{
			$<strtime>$=(char *)calloc(1,20);	
			(void) sprintf($<strtime>$,"%ld",$<integer>1);
			}
		;

date		:	integer months integer
			{
			struct tm timing;

			(void) memset(&timing,'\000',sizeof(timing));
			timing.tm_mday=$<integer>1;
			timing.tm_mon=$<integer>2;
	 		if ($<integer>3<100)
			  $<integer>3+=2000;
			timing.tm_year=$<integer>3-1900;
			$<time>$=timegm(&timing);
			}
		;

time		:	hours zone
			{
			$<time>$=$<time>1-$<time>2;
			}
		|	hours
			{
			$<time>$=$<time>1;
			}
		|	zone
			{
			$<time>$=-$<time>1;
			}
		;

hours		:	hour COLON sixty COLON sixty
			{
			$<time>$=((($<time>1*60)+($<time>3))*60)+$<time>5;
			}
		|	hour COLON sixty
			{
			$<time>$=(($<time>1*60)+($<time>3))*60;
			}
		;

hour		:	integer
			{
			$<time>$=$<integer>1;
			if ($<integer>1>24) {
			  (void) yyerror("Unexpected value (above 24 Hour)");
			  $<time>$=0;
			  } 
			}
		;

sixty		:	integer
			{
			$<time>$=$<integer>1;
			if ($<integer>1>60) {
			  (void) yyerror("Unexpected value (above 60)");
			  $<time>$=0;
			  } 
			}
		;

zone		:	sign integer optime
			{
			$<time>$=(($<integer>2/100)*3600)+(($<integer>2%100)*60);
			$<time>$*=$<integer>1;
			}
		|	WORD
			{
			static struct {
				char *zname;
				int diff;
				}tznom[]={
					  {"UTC",-0},
					  {"Z",-0},
					  {"GMT",-0},
					  {"EST",-5},
					  {"EDT",-4},
					  {"CST",-6},
					  {"CDT",-5},
					  {"MST",-7},
					  {"MDT",-6},
					  {"PST",-8},
					  {"PDT",-7},
					  {"AKST",-9},
					  {"AKDT",-8},
					  {"HST",-10},
					  {"HST",-10},
					  {"BST",+1},
					  {"CET",+1},
					  {"CST",+2},
					  {(char *)0,0}
					  };
			int i;

			$<time>$=0;
			for (i=0;tznom[i].zname!=(char *)0;i++) {
			  if (strcasecmp(tznom[i].zname,$<word>1)==0) {
			    $<time>$=tznom[i].diff*3600;
			    break;
			    }
			  }	
			(void) free($<word>1);
			}
		;

optime		:	PARLEFT comments PARRIGHT
		|
		;


comments	:	comment
		|	comments comment
		;

comment		:	word
			{
			(void) free($<word>1);
			}
		|	AT
		;
/*

*/
/************************************************/
/*	E-Mail address definition		*/
/************************************************/
address		:	mailboxlist
			{
			$<mboxes>$=$<mboxes>1;
			}
		|	group
			{
			$<mboxes>$=$<mboxes>1;
			}
		|
			{
			$<mboxes>$=(char **)0;
			}
		;

group		:	words COLON mailboxlist SEMICOL
			{
			if ($<word>1!=(char *)0)
			  (void) free($<word>1);
			$<mboxes>$=$<mboxes>3;
			}
		|	words COLON SEMICOL
			{
			if ($<word>1!=(char *)0)
			  (void) free($<word>1);
			$<mboxes>$=(char **)0;
			}
		|	words COLON mailboxlist 
			{
			if ($<word>1!=(char *)0)
			  (void) free($<word>1);
			$<mboxes>$=$<mboxes>3;
			}
		;

mailboxlist	:	mailbox
			{
			$<mboxes>$=(char **)0;
			if ($<email>1!=(char *)0) {
			  (void) rou_strtolower($<email>1);
  			  $<mboxes>$=(char **)rou_addveclst((void **)0,
						           (void *)$<email>1);
			  }
			}
		|	mailboxlist COMMA mailbox
			{
			$<mboxes>$=$<mboxes>1;
			if ($<email>1!=(char *)0) {
			  (void) rou_strtolower($<email>3);
  			  $<mboxes>$=(char **)rou_addveclst((void **)$<mboxes>1,
						           (void *)$<email>3);
			  }
			}
		;

mailbox		:	addrspec
			{
			$<email>$=$<email>1;
			}
		|	addrspec routeaddr
			{
			if ($<email>1!=(char *)0)
			  (void) free($<email>1);
			$<email>$=$<email>2;
			}
		|	routeaddr
			{
			/*trapping common syntax error	*/
			$<email>$=$<email>1;
			}
		|	routeaddr words
			{
			/*trapping common syntax error	*/
			if ($<email>2!=(char *)0)
			  (void) free($<email>2);
			$<email>$=$<email>1;
			}
		|	addrspec error
			{
			$<email>$=$<email>1;
			}
		|	words routeaddr
			{
			if ($<email>1!=(char *)0)
			  (void) free($<email>1);
			$<email>$=$<email>2;
			}
		|	localpart
			{
			$<email>$=$<email>1;
			}
		;

addrspec	:	localpart AT domain
			{
			char *strloc;

			strloc=strmix($<email>1,'@',$<email>3);
			(void) rou_strtolower(strloc);
			$<email>$=strloc;
			}
		/*this is an error	*/
		|	localpart AT
			{
			if ($<word>1!=(char *)0)
			  (void) free($<word>1);
			$<word>$=(char *)0;
			}
		;

localpart	:	words
			{
			$<email>$=$<email>1;
			}
		|	localpart DOT words
			{
			$<email>$=strmix($<email>1,'.',$<word>3);
			}
		|       localpart DOT error
			{
			$<email>$=$<email>1;
			}
		|	localpart DOT error words
			{
			if ($<word>4!=(char *)0) {
			  (void) free($<word>4);
			  }
			$<email>$=$<email>1;
			}
		;

domain		:	subdomain
			{
			$<email>$=$<email>1;
			}
		|	domain DOT subdomain
			{
			$<email>$=strmix($<email>1,'.',$<email>3);
			}
		|	domain DOT error
			{
			$<email>$=$<email>1;
			}
		;

subdomain	:	domainref
			{
			$<email>$=$<email>1;
			}
		|	domainliteral
			{
			$<email>$=$<email>1;
			}
		;

domainref	:	atom
			{
			$<email>$=$<word>1;
			}
		|	domainref atom
			{
			$<email>$=strmix($<email>1,'\000',$<word>2);
			}
		;

domainliteral	:	BRKLEFT words BRKRIGHT
			{
			$<email>$=(char *)0;
			if ($<word>2!=(char *)0) {
			  $<email>$=(char *)calloc(1,strlen($<word>2)+4);
			  (void) strcpy($<email>$,"[");
			  (void) strcat($<email>$,$<word>2);
			  (void) strcat($<email>$,"]");
			  (void) free($<word>2);
			  }
			}
		;

routeaddr	:	LESSTHAN route addrspec GRTRTHAN
			{
			$<email>$=$<email>3;
			}
		|	LESSTHAN addrspec GRTRTHAN
			{
			$<email>$=$<email>2;
			}
		|	LESSTHAN GRTRTHAN
			{
			$<email>$=(char *)0;
			}
		|	LESSTHAN words GRTRTHAN
			{
			if ($<word>2!=(char *)0)
			  (void) free($<word>2);
			nbrerr++;
			$<email>$=(char *)0;
			}
		;

route		:	AT domain COLON
			{
			if ($<email>2!=(char *)0)
			  (void) free($<word>2);
			}
		;

/************************************************/
/*	Low Level language definition.		*/
/************************************************/
words		:	word
			{
			$<word>$=$<word>1;
			}
		|	words word
			{
			$<word>$=strmix($<word>1,'\000',$<word>2);
			}
		|	words error
			{
			if ($<word>1!=(char *)0) {
			  (void) free($<word>1);
			  }
			$<word>$=(char *)0;
			}
		;

word		:	atom
			{
			$<word>$=$<word>1;
			}
		|	STRING
			{
			$<word>$=$<word>1;
			}
		|	DEBENCOD 
			{	/*WAITING for end of encoding	*/
			int phase;
			int proceed;

			phase=0;
			proceed=true;
			while (skipfor('?')==true) {
			  switch (phase) {
			    case 0  :
			    case 1  :
			      phase++;
			      break;
			    case 2  :
			      if (lines[numline][numrow+1]!='=') {
			        nbrerr++;
			        (void) yyerror("Encoding syntax error");
				}
			      proceed=false;
			      break;
			    }
			  if (proceed==true)
			    numrow+=1;
			  else
			    break;
			  }
			}
				ENCODFIN
			{
			$<word>$=(char *)0;
			}
		;

integer		:	WORD
			{
			char *ptr;

			ptr=$<word>1;
			$<integer>$=0;
        		if (strchr(ptr,'.')==(char *)0) {
			  long lval;

          		  if (sscanf(ptr,"%ld",&lval)==1) {
            		    $<integer>$=lval;
            		    }
          		  }
        		else {
			  (void) yyerror("Integer expected, found float");
			  }
			(void) free($<word>1);
			}
		;

atom		:	WORD
			{
			$<word>$=$<word>1;
			}
		|	others
			{
			$<word>$=$<word>1;
			}
		;


others		:	punctuation
			{
			$<word>$=$<word>1;
			}
		|	marker
			{
			$<word>$=$<word>1;
			}
		|	days
			{
		        static char *daylist[]={
		   		"Sun","Mon","Tue","Wed","Thu",
		   		"Fri","Sat"};
			$<word>$=strdup(daylist[$<integer>1]);
			}
		|	months
			{
		        static char *monthlist[]={
				"Jan","Feb","Mar","Apr",
				"May","Jun","Jul","Aug",
		   		"Sep","Oct","Nov","Dec"};
			$<word>$=strdup(monthlist[$<integer>1]);
			}
		;

punctuation	:	PUNCT
			{
			char strloc[10];

			(void) memset(strloc,'\000',sizeof(strloc));
			strloc[0]=$<punct>1;
			$<word>$=strdup(strloc);
			}
		|	MINUS
			{
			$<word>$=strdup("-");
			}
		|	PLUS
			{
			$<word>$=strdup("+");
			}
		|	QUESTION
			{
			$<word>$=strdup("?");
			}
		;

marker		:	DATE
			{
			$<word>$=strdup("Date");
			}
		|	TO
			{
			$<word>$=strdup("To");
			}
		|	CC
			{
			$<word>$=strdup("Cc");
			}
		|	BCC
			{
			$<word>$=strdup("Bcc");
			}
		|	FROM
			{
			$<word>$=strdup("From");
			}
		|	SENDER
			{
			$<word>$=strdup("Sender");
			}
		|	REPLYTO
			{
			$<word>$=strdup("Reply-To");
			}
		;

months		:	JAN
			{
			$<integer>$=0;
			}
		|	FEB
			{
			$<integer>$=1;
			}
		|	MAR
			{
			$<integer>$=2;
			}
		|	APR
			{
			$<integer>$=3;
			}
		|	MAY	
			{
			$<integer>$=4;
			}
		|	JUN
			{
			$<integer>$=5;
			}
		|	JUL
			{
			$<integer>$=6;
			}
		|	AUG
			{
			$<integer>$=7;
			}
		|	SEP
			{
			$<integer>$=8;
			}
		|	OCT
			{
			$<integer>$=9;
			}
		|	NOV
			{
			$<integer>$=10;
			}
		|	DEC
			{
			$<integer>$=11;
			}
		;

days		:	SUN
			{
			$<integer>$=0;
			}
		|	MON
			{
			$<integer>$=1;
			}
		|	TUE
			{
			$<integer>$=2;
			}
		|	WED
			{
			$<integer>$=3;
			}
		|	THU
			{
			$<integer>$=4;
			}
		|	FRI
			{
			$<integer>$=5;
			}
		|	SAT
			{
			$<integer>$=6;
			}
		;

sign		:	MINUS
			{
			$<integer>$=-1;
			}
		|	PLUS
			{
			$<integer>$=+1;
			}
		|	PLUS MINUS
			{
			$<integer>$=+1;
			}
		;
%%
#include	<memory.h>
#include	<ctype.h>
#include	<errno.h>

typedef	enum	{	/*email directive	*/
	mh_orgn,	/*E-mail physic origine	*/
	mh_from,	/*E-mail sender name	*/
	mh_rcpt,	/*E-mail recipient	*/
	mh_data		/*E-mail data phase	*/
	}MHTYP;

/*	Token list				*/
static struct	{
	int code;
	char *string;
	}vocable[]={	/*string MUST be in length order	*/
		   {INRPLYTO,	"In-Reply-To"},
		   {RDATE,	"Resent-Date"},
		   {MSGID,	"Message-ID"},
		   {RFERENCE,	"References"},
		   {REPLYTO,	"Reply-To"},
		   {SUBJECT,	"Subject"},
		   {SENDER,	"Sender"},
		   {DATE,	"Date"},
		   {FROM,	"From"},
		   {BCC,	"Bcc"},
		   {MON,"Mon"},{TUE,"Tue"},{WED,"Wed"},{THU,"Thu"},
		   {FRI,"Fri"},{SAT,"Sat"},{SUN,"Sun"},
		   {JAN,"Jan"},{FEB,"Feb"},{MAR,"Mar"},{APR,"Apr"},
		   {MAY,"May"},{JUN,"Jun"},{JUL,"Jul"},{AUG,"Aug"},
		   {SEP,"Sep"},{OCT,"Oct"},{NOV,"Nov"},{DEC,"Dec"},
		   {TO,		"To"},
		   {CC,		"Cc"},
		   {WORD,(char *)0}
		   };
/*

*/
/************************************************/
/*						*/
/*	Procedure to allocate a memory being	*/
/*	the contents of 2 memories, separated	*/
/*	by charactere (free the 2 previously	*/
/*	allocated memories)			*/
/*						*/
/************************************************/
static char *strmix(char *part1,char sep,char *part2)

{
char *mix;
int taille;

mix=(char *)0;
if (part1==(char *)0)
  part1=strdup("");
if (part2==(char *)0)
  part2=strdup("");
taille=strlen(part1)+strlen(part2)+2;
mix=(char *)calloc(1,taille);
if (sep!='\000') 
  (void) snprintf(mix,taille,"%s%c%s",part1,sep,part2);
else
  (void) snprintf(mix,taille,"%s%s",part1,part2);
(void) free(part2);
(void) free(part1);
return mix;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to report a parsing error	*/
/*	debugging mode only.			*/
/*						*/
/************************************************/
static void yyerror(char *info)

{
int i;
char crlf[3];

(void) strcpy(crlf,"");
for (i=0;lines[curline][i]!='\000';i++)
  if (isspace(lines[curline][i])!=false)
    lines[curline][i]=' ';
if (currow>=strlen(lines[curline]))
  (void) strcpy(crlf,"\\n");
(void) tracing(TRALOG,0,"Parsing header Trouble!");
(void) tracing(TRALOG,0,"% 6d %.*s",curline+1,currow+2,lines[curline]);
(void) tracing(TRALOG,0,"%*s %*s%s",6,"",currow+1,"^",info);
nbrerr++;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract the punct char	*/
/*	value.					*/ 	
/*						*/
/************************************************/
static int punctchar(char c)

{
int code;

code=PUNCT;
switch (c) {
  case '\n'	:
    code=CRLF;
    break;
  case ':'	:
    code=COLON;
    break;
  case ';'	:
    code=SEMICOL;
    break;
  case ','	:
    code=COMMA;
    break;
  case '@'	:
    code=AT;
    break;
  case '.'	:
    code=DOT;
    break;
  case '-'	:
    code=MINUS;
    break;
  case '+'	:
    code=PLUS;
    break;
  case '('	:
    code=PARLEFT;
    break;
  case ')'	:
    code=PARRIGHT;
    break;
  case '['	:
    code=BRKLEFT;
    break;
  case ']'	:
    code=BRKRIGHT;
    break;
  case '<'	:
    code=LESSTHAN;
    break;
  case '>'	:
    code=GRTRTHAN;
    break;
  case '='	:
    code=PUNCT;
    if (numrow!=0) {
      if (lines[numline][numrow]=='?') {
        numrow++;
        code=DEBENCOD;
        }
      }
    break;
  case '?'	:
    code=QUESTION;
    if (numrow!=0) {
      if (lines[numline][numrow]=='=') {
        numrow++;
        code=ENCODFIN;
        }
      }
    break;
  default	:
    code=PUNCT;
    break;
  }
return code;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract the token value 	*/
/*	from a string				*/
/*						*/
/************************************************/
static int tokenfromstr(char *strtoken,int *disp)

{
int token;
int i;

*disp=0;
token=WORD;
for (i=0;vocable[i].string!=(char *)0;i++) {
  *disp=strlen(vocable[i].string);
  if (strncasecmp(vocable[i].string,strtoken,*disp)==0) {
    token=vocable[i].code;
    break;
    }
  }
return token;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to backspace in the header	*/
/*	the data-header				*/
/*						*/
/************************************************/
static void backspace()

{
numline=curline;
numrow=currow;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get the next character from*/
/*	the data-header				*/
/*						*/
/************************************************/
static char getnextchar()

{
int c;

c=EOF;
if (lines[numline]!=(char *)0) {
  curline=numline;
  currow=numrow;
  if (numrow==0)
    (void) tracing(TRALOG,9,"%6d line=<%s>",numline+1,lines[numline]);
  c=(int)lines[numline][numrow];
  if (c=='\000') {
    c='\n';
    numrow=0;
    numline++;
    }
  else
    numrow++;
  }
return c;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to synchronize on  a special	*/
/*	character within the line parsed line.	*/
/*						*/
/************************************************/
static int skipfor(char expected)

{
int found;

found=false;
while (lines[numline][numrow]!='\000') {
  if (lines[numline][numrow]==expected) {
    found=true;
    break;
    }
  numrow++;
  }
return found;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract a string on a line	*/
/*						*/
/************************************************/
static char *getstring()

{
#define STRINC  20

char c;
char *strloc;
int locnumline;
int locnumrow;

locnumline=numline;
locnumrow=numrow;
strloc=(char *)calloc(1,STRINC+5);
while ((c=getnextchar())!=EOF) {
  int bspace;
  char ajout[2];

  bspace=false;
  if ((strlen(strloc)%STRINC)==0) {
    strloc=(char *)realloc(strloc,(((strlen(strloc)/STRINC)+1)*STRINC)+5);
    }
  bspace=((strlen(strloc)>1)&&(strloc[strlen(strloc)]=='\\'));
  if ((c=='\n')&&(bspace==false)) {
    (void) free(strloc);
    strloc=(char *)0;
    numline=locnumline;
    numrow=locnumrow;
    break;
    }
  if ((c=='\"')&&(bspace==false))
    break;
  ajout[0]=c;
  (void) strncat(strloc,ajout,1);
  }
return strloc;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to skip comment in the header	*/
/*						*/
/************************************************/
static int skipcomment()

{
int c;
int proceed;

proceed=true;
c='(';
while (proceed==true) {
  proceed=false;
  c=getnextchar();
  if (c=='(')
    c=skipcomment(); 
  switch(c) {
    case ')'	:
      c=getnextchar();
      break;
    case '\n'	:
    case EOF	:
      (void) yyerror("')' expected ");
      break;
    default	:
      proceed=true;
      break;
    }
  }
return c;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to skip all blank character	*/
/*						*/
/************************************************/
static int skipblank()

{
int c;
int proceed;

c=EOF;
proceed=true;
while (proceed==true) {
  c=getnextchar();
  while (c=='(') {
    c=skipcomment();
    }
  if (isblank(c)==false)
    proceed=false;
  }
return c;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get/recognise a lexical	*/
/*	input.					*/
/*						*/
/************************************************/
static int yylex()

{
#define MXINFO	200

int found;
int c;
int proceed;
int phase;
int disp;
int numerique;
char info[MXINFO];

found=STOP;
phase=0;
proceed=true;
disp=0;
numerique=false;
(void) memset(info,'\000',sizeof(info));
c=skipblank();
if (c==EOF) {
  if (cptstop>0)
    (void) siglongjmp(manystop,1);
    cptstop++;
  return STOP;
  }
while (proceed==true) {
  switch (phase) {
    case     0	:	/*looking for strings	*/
      if (c=='\"') {
        if ((yylval.string=getstring())!=(char *)0) {
	  found=STRING;
	  proceed=false;
	  }
	}
      break;
    case     1	:	/* looking for punct	*/
      if (isalnum(c)==false) {
        found=punctchar(c);
	yylval.punct=(char)c;
	proceed=false;
	}
      break;
    case     2	:	/* looking for keyword	*/
      if ((found=tokenfromstr(lines[numline]+(numrow-1),&disp))!=WORD) {
	char *ptr;

	ptr=lines[numline]+(numrow-1)+disp;
	if (isalnum(*ptr)==false) {
	  numrow+=(disp-1);
	  proceed=false;
	  }
	}
      break;
    case     3	:	/*checking if word could be numeric	*/
      if (isdigit(c)!=false) 
	numerique=true;
      break;
    case     4	:	/*extracting word			*/
      {
      int extract;
      int inx;

      extract=true;
      inx=0;
      while (extract==true) {
        info[inx++]=c;
        if ((c=getnextchar())==EOF) {
          found=STOP;
	  if (cptstop>0)
	    (void) siglongjmp(manystop,1);
	  cptstop++;
	  extract=false;
	  break;
          }
        if (inx>=sizeof(info))
          extract=false;
        if (extract==false)
	  break;
        extract=((isalnum(c)!=false));
        if (numerique==true)
	  extract|=((c=='.')||(c=='-')||(c=='+'));
	}
      (void) backspace();
      }
      break;
    case     5	:	/* this an alphanumic word	*/
      yylval.word=strdup(info);
      found=WORD;
      proceed=false;
      break;
    default	:	/*safe guard			*/
      proceed=false;
      break;
    }
  phase++;
  }
#ifdef DEBUG
if (yydebug>0) {
  /*synchronizing yydebug output	*/
  (void) fflush(stdout);
  (void) fflush(stderr);
  }
#endif
if (debug>=11)
  (void) tracing(TRALOG,0,"return token (value='%d')",found);
return found;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free the memory used by	*/
/*	a HDRTYP ** structure			*/
/*						*/
/************************************************/
static HDRTYP **cleanhdr(HDRTYP **chdrs)

{
if (chdrs!=(HDRTYP **)0) {
  int i;

  for (i=0;chdrs[i]!=(HDRTYP *)0;i++) {
    switch (chdrs[i]->field) {
      case (hd_from)		:
      case (hd_sender)		:
      case (hd_replyto)		:
      case (hd_msgid)		:
        if (chdrs[i]->info.address!=(char *)0)
          (void) free(chdrs[i]->info.address);
	break;
      case (hd_cc)		:
      case (hd_bcc)		:
      case (hd_to)		:
      case (hd_inreplyto)	:
      case (hd_references)	:
        if (chdrs[i]->info.maddr!=(char **)0) {
	  int j;

	  for (j=0;chdrs[i]->info.maddr[j]!=(char *)0;j++) {
	    (void) free(chdrs[i]->info.maddr[j]);
	    }
          (void) free(chdrs[i]->info.maddr);
	  }
	break;
      case (hd_date)		:
        if (chdrs[i]->info.time!=(char *)0)
          (void) free(chdrs[i]->info.time);
	break;
      case (hd_subject)		:
        if (chdrs[i]->info.subject!=(char *)0)
          (void) free(chdrs[i]->info.subject);
	break;
      default			:
        (void) tracing(TRALOG,0,"unievl.y,hdr_cleanhdr: Unexpected field (bug?)");
	break;
      }
    (void) free(chdrs[i]);
    }
  (void) free(chdrs);
  chdrs=(HDRTYP **)0;
  }
return chdrs;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to trace the address part of	*/
/*	header contents				*/
/*						*/
/************************************************/
static void displayaddr(int lv,HDRTYP *hdr)

{
char *ptr;

switch(hdr->field) {
  case hd_to		:
    ptr="To";
    break;
  case hd_cc		:
    ptr="Cc";
    break;
  case hd_bcc		:
    ptr="Bcc";
    break;
  case hd_inreplyto	:
    ptr="In-Reply-To";
    break;
  case hd_references	:
    ptr="References";
    break;
  default		:
    ptr="(bug?)";
    break;
  }
if (hdr->info.maddr!=(char **)0) {
  int i;

  for (i=0;hdr->info.maddr[i]!=(char *)0;i++) 
    (void) tracing(TRALOG,lv,"E_%s: <%s>",ptr,hdr->info.maddr[i]);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to trace the wanted headers	*/
/*	contents, in the debug logs		*/
/*						*/
/************************************************/
static void tracehdr(int lv,HDRTYP **chdrs)

{
if (chdrs!=(HDRTYP **)0) {
  int i;

  for (i=0;chdrs[i]!=(HDRTYP *)0;i++) {
    switch (chdrs[i]->field) {
      case (hd_from)		:
	(void) tracing(TRALOG,lv,"E_From: <%s>",chdrs[i]->info.address);
	break;
      case (hd_cc)		:
      case (hd_bcc)		:
      case (hd_to)		:
      case (hd_inreplyto)	:
      case (hd_references)	:
	(void) displayaddr(lv,chdrs[i]);
	break;
      case (hd_sender)		:
	(void) tracing(TRALOG,lv,"E_Sender: <%s>",chdrs[i]->info.address);
	break;
      case (hd_replyto)		:
	(void) tracing(TRALOG,lv,"E_Reply-To: <%s>",chdrs[i]->info.address);
	break;
      case (hd_msgid)		:
	(void) tracing(TRALOG,lv,"E_Message-ID: <%s>",chdrs[i]->info.address);
	break;
      case (hd_date)		:
	{
	char *ptr;
	time_t temps;

	temps=atol(chdrs[i]->info.time);	
	ptr=asctime(gmtime(&temps));
	ptr[strlen(ptr)-1]='\000';
	(void) tracing(TRALOG,lv,"E_Date: <%s>",ptr);
	}
	break;
      case (hd_subject)		:
	(void) tracing(TRALOG,lv,"E_Subject: <%s>",chdrs[i]->info.subject);
	break;
      default			:
        (void) tracing(TRALOG,0,"unievl.c,evl_tracehdr: Unexpected "
				   "field (bug?)");
	break;
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to scan a header and locate	*/
/*	specific information			*/
/*						*/
/************************************************/
static HDRTYP **mkhdr(char **data)

{
int verybad;

verybad=false;
cptstop=0;
ghdrs=(HDRTYP **)0;
lines=data;
numline=0;
numrow=0;
itsok=true;
nbrerr=0;
#ifdef DEBUG
if (debug>11)	/*debugging all yacc analyze	*/
  yydebug=1;
#endif
if (sigsetjmp(manystop,true)==0)
  verybad=yyparse(); 
else {
  verybad=true;
  }
if (verybad==true)
  ghdrs=cleanhdr(ghdrs); 
/*to fully clean yyparse used memory		*/
/*
if (yyvs!=(YYSTYPE *)0) {
  (void) free(yyvs);
  yyvs=(YYSTYPE *)0;
  }
if (yyss!=(short *)0) {
  (void) free(yyss);
  yyss=(short *)0;
  }
*/
return ghdrs;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract a reference found	*/
/*	available in the E-mail header.		*/
/*						*/
/************************************************/
static void *gethdrinfo(HDRTYP **hdrs,HIDTYP field)


{
void *str;

str=(char *)0;
if (hdrs!=(HDRTYP **)0) {
  int i;

  for (i=0;hdrs[i]!=(HDRTYP *)0;i++) {
    if (hdrs[i]->field==field) {
      switch (field) {
        case (hd_from)		:
        case (hd_sender)	:
        case (hd_replyto)	:
        case (hd_msgid)		:
          str=(void *)hdrs[i]->info.address;
	  break;
  	case (hd_date)		:
          str=(void *)hdrs[i]->info.time;
	  break;
  	case (hd_inreplyto)	:
  	case (hd_references)	:
          str=(void *)hdrs[i]->info.maddr;
	  break;
  	case (hd_to)		:
  	case (hd_cc)		:
  	case (hd_bcc)		:
          str=(void *)hdrs[i]->info.maddr;
	  break;
        case (hd_subject)	:
          str=(void *)hdrs[i]->info.subject;
	  break;
  	default			:
    	  (void) tracing(TRALOG,0,"unievl.c,evl_genheader: Unexpected field (bug?)");
	  break;
	}
      break;
      }
    }
  }
return str;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to collect specific set of	*/
/*	headers.				*/
/*						*/
/************************************************/
static char **mklist(HDRTYP **hdrs,char **list,HIDTYP hid)

{
char **mptr;

if ((mptr=gethdrinfo(hdrs,hid))!=(char **)0) {
  int i;

  for (i=0;mptr[i]!=(char *)0;i++) {
    list=(char **)rou_addveclst((void **)list,(void *)strdup(mptr[i]));
    }
  }
return list;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to normalize some E-mails	*/
/*	(generated by mailling list) such	*/
/*	address x.12.34@somewhere.com		*/
/*	is translated to x@somewhwere.com	*/
/*						*/
/************************************************/
static char *cleanreal(char *real)

{
if (real!=(char *)0) {
  char *ptr;

  if ((ptr=strchr(real,'.'))!=(char *)0) {
    char *dotptr;
    int skip;
    int proceed;

    dotptr=ptr;
    ptr++;
    skip=true;
    proceed=true;
    while (proceed==true) {
      switch (*ptr) {
        case '@'  	:
        case '\000'  	:
	  proceed=false;	/*do NOT break	*/
        case '.'	:
	  if (skip==true) {
	    (void) strcpy(dotptr,ptr);
	    ptr=dotptr;
	    }
	  dotptr=ptr;
	  skip=true;
	  break;
	default		:
	  if (skip==true)
	    skip=(isdigit(*ptr)!=false);
	  break;
        }
      ptr++;
      }
    }
  }
return real;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to look at E-mail header	*/
/*	information and extract selected fields	*/
/*						*/
/************************************************/
static EVLTYP *lookhdrs(HDRTYP **hdrs)

{
int phase;
int proceed;
EVLTYP *evl;

proceed=true;
phase=1;	/*phase 0 is the exist phase	*/
evl=(EVLTYP *)calloc(1,sizeof(EVLTYP));
while (proceed==true) {
  char *ptr;

  ptr=(char *)0;
  switch (phase) {
    case	 0  :
      proceed=false;
      break;
    case	 1  :	/*extracting "Sender"	*/
      if ((ptr=gethdrinfo(hdrs,hd_sender))!=(char *)0) {
	evl->org=org_sender;
        evl->realfrom=strdup(ptr);
	phase+=2;	/*getting MSG DATE	*/
        }
      break;
    case	 2  :	/*extracting "Reply-To"	*/
      if ((ptr=gethdrinfo(hdrs,hd_replyto))!=(char *)0) {
	evl->org=org_replyto;
        evl->realfrom=strdup(ptr);
	phase+=1;	/*getting MSG DATE	*/
        }
      break;
    case	 3  :	/*extracting "From"	*/
      if ((ptr=gethdrinfo(hdrs,hd_from))!=(char *)0) {
	evl->org=org_from;
        evl->realfrom=strdup(ptr);
        }
      break;
    case	 4  :	/*empty phase 		*/
      break;
    case	 5  :	/*extracting MSG date	*/
      if ((ptr=gethdrinfo(hdrs,hd_date))!=(char *)0) {
        evl->cachet=(time_t)atol(ptr);
        }
      break;
    case	 6  :	/*extracting MSGID	*/
      if ((ptr=gethdrinfo(hdrs,hd_msgid))!=(char *)0) {
        evl->msgid=strdup(ptr);
	}
      break;
    case	 7  :	/*extracting Subject	*/
      if ((ptr=gethdrinfo(hdrs,hd_subject))!=(char *)0) {
        evl->subject=strdup(ptr);
	}
      break;
    case	 8  :	/*extracting In-Reply-To*/
      evl->inrplyto=mklist(hdrs,(char **)0,hd_inreplyto);
      evl->inrplyto=mklist(hdrs,evl->inrplyto,hd_references);
      break;
    case	 9  :	/*extracting To,Cc,Bcc	*/
      evl->distlist=mklist(hdrs,(char **)0,hd_to);
      evl->distlist=mklist(hdrs,evl->distlist,hd_cc);
      evl->distlist=mklist(hdrs,evl->distlist,hd_bcc);
      break;
    case	10  :	/*cleaning realfrom	*/
      evl->realfrom=cleanreal(evl->realfrom); 
      break;
    default	    :	/*fail safe		*/
      phase=-1;		/*going to the exit	*/
      break;
    }
  phase++;
  }
return evl;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to build a E-mail header	*/
/*	list, by concatanating lines which start*/
/*	with a space (see RFC 2022)		*/
/*						*/
/************************************************/
char **mkyachdr(char **headers)

{
char **yachdr;

yachdr=(char **)0;
if (headers!=(char **)0) {
  int i;

  for (i=0;headers[i]!=(char *)0;i++) {
    char *ptr;

    ptr=strdup(headers[i]);
    while (headers[i+1]!=(char *)0) {
      if (isblank(headers[i+1][0])==0)
	break;
      i++;
      ptr=(char *)realloc(ptr,strlen(ptr)+strlen(headers[i])+4);
      (void) strcat(ptr,headers[i]);
      }
    yachdr=(char **)rou_addveclst((void **)yachdr,(void *)ptr);
    }
  }
return yachdr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to analyse the email header	*/
/*	part. looking for information as:	*/
/*	Sender:, Reply-To:, From:...		*/
/*						*/
/************************************************/
EVLTYP *hdr_parseheader(char *session,char **headers)

{
EVLTYP *evl;

evl=(EVLTYP *)0;
if (headers!=(char **)0) {
  HDRTYP **hdrs;
  char **yachdr;

  yachdr=mkyachdr(headers);
  if ((hdrs=mkhdr(yachdr))!=(HDRTYP **)0) {
    evl=lookhdrs(hdrs);
    evl->nbrerror=nbrerr;
    if (debug>=2)
      (void) tracehdr(2,hdrs);
    hdrs=cleanhdr(hdrs);
    }
  else {
    int i;

    (void) journalling(true,APJRL,"fatal header parsing error! (session=<%s>)",
				   session);
    (void) alert(0,"geshdr.y:hdr_parsehdr, fatal header parsing error "
		   "(session=<%s>",session);
    for (i=0;headers[i]!=(char *)0;i++) {
      (void) alert(0,"header[%d]=<%s>",i,headers[i]);
      }
    }
  (void) rou_freestrlst(yachdr);
  }
return evl;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by an	*/
/*	E-mail info structure			*/
/*						*/
/************************************************/
EVLTYP *hdr_freeevl(EVLTYP *evl)

{
if (evl!=(EVLTYP *)0) {
  evl->spamlvl=rou_freestr(evl->spamlvl);
  evl->virname=rou_freestr(evl->virname);
  evl->distlist=rou_freestrlst(evl->distlist);
  evl->inrplyto=rou_freestrlst(evl->inrplyto);
  evl->msgid=rou_freestr(evl->msgid);
  evl->subject=rou_freestr(evl->subject);
  evl->realfrom=rou_freestr(evl->realfrom);
  (void) free(evl);
  evl=(EVLTYP *)0;
  }
return evl;
}
