/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	GESTLS:					*/
/*	Take care of all TSL channel management	*/
/*						*/
/************************************************/
#include	<string.h>
#include	<syslog.h>
#include	<unistd.h>
#include	<stdarg.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<netdb.h>
#include	<setjmp.h>
#include	<sys/poll.h>

#define		OPENSSL_NO_KRB5
#include 	<openssl/x509.h>
#include 	<openssl/x509v3.h>
#include 	<openssl/safestack.h>
#include	<openssl/ssl.h>
#include	<openssl/err.h>

#include	"gestls.h"
#include	"unieml.h"
#include	"subrou.h"

#define	MXBUF	16384	/*char in buffer size	*/
#define	MXTRY	30	/*30 second max to 	*/
			/*establish an SSL	*/
			/*connexion		*/

typedef	struct	{
	int tls;	/*connexion en mode TLS	*/
	int handle;	/*direct link handle	*/
	time_t lastused;/*last time handle used	*/
	int goteof;	/*EOF detected		*/
	int incpt;	/*# char in the buffer	*/
	char *carin;	/*in char buffer	*/
	char *cipherid;	/*SSL version connection*/
	SSL_CTX *ctx;	/*context encryption	*/
	SSL *ssl;	/*ref connection TLS	*/
	BIO *bio;	/*SSL basic I/O		*/
	}TFILE;

static	jmp_buf	toolong;
/*

*/
/************************************************/
/*						*/
/*	Setting up the CTX information for	*/
/*	the connection.				*/
/*						*/
/************************************************/
static SSL_CTX *setup_tls_ctx()

{
#define TRUSTED		"trusted"
#define SSL_CIPHER_LIST	"ALL:!LOW"

SSL_CTX *ctx;
int mode;
int phase;
int proceed;
char certdir[300];
char cafile[300];
char pemfile[300];

ctx=(SSL_CTX *)0;
mode=SSL_VERIFY_NONE;
phase=1;
proceed=true;
(void) rou_mkname(certdir,sizeof(certdir),d_tls,"","","");
(void) snprintf(cafile,sizeof(cafile),"%s/%s.pem",certdir,TRUSTED);
(void) snprintf(pemfile,sizeof(pemfile),"%s/%s.pem",certdir,appname);
while (proceed==true) {
  char *errmsg;
  STACK_OF(X509_NAME) *c_names;

  errmsg=(char *)0;
  c_names=(STACK_OF(X509_NAME) *)0;
  switch (phase) {
    case  0	:	/*Displaying and aborting*/
      if (ctx!=(SSL_CTX *)0) {
	(void) SSL_CTX_free(ctx);
	ctx=(SSL_CTX *)0;
        }
      proceed=false;
      break;
    case  1	:	/*getting a CTX	data	*/
      if ((ctx=SSL_CTX_new(SSLv23_method()))==(SSL_CTX *)0) {
	errmsg="CTX_new";
        phase=-phase;
	}
      break;
    case  2	:	/*loading CA verify dir	*/
      if (SSL_CTX_load_verify_locations(ctx,cafile,(char *)0)==0) {
	errmsg="verify_location";
        phase=-phase;
        }
      break;
    case  3	:	/*loading CA verify dir	*/
      if (SSL_CTX_set_default_verify_paths(ctx)==0) {
	errmsg="verify_paths";
        phase=-phase;
        }
      break;
    case  4	:	/*loading appl certificat*/
      if (SSL_CTX_use_certificate_chain_file(ctx,pemfile)!=1) {
	errmsg="use_certificat";
        phase=-phase;
        }
      break;
    case  5	:	/*loading appl certificat*/
      if (SSL_CTX_use_PrivateKey_file(ctx,pemfile,SSL_FILETYPE_PEM)!=1) {
	errmsg="use_privatekey";
        phase=-phase;
        }
      break;
    case  6	:	/*verify management	*/
      (void) SSL_CTX_set_purpose(ctx,X509_PURPOSE_ANY);
      (void) SSL_CTX_set_verify(ctx,mode,(int(*)())0);
      (void) SSL_CTX_set_verify_depth(ctx,5);
      (void) SSL_CTX_set_options(ctx,SSL_OP_ALL);
      if (SSL_CTX_set_cipher_list(ctx,SSL_CIPHER_LIST)==0) {
	errmsg="set_cipher";
        phase=-phase;
        }
      break;
    case  7	:	/*allowing partial write*/
      (void) SSL_CTX_set_mode(ctx,SSL_MODE_ENABLE_PARTIAL_WRITE);
      break;
    default	:	/*SAFE guard		*/
      proceed=false;
      break;
    }
  if (phase<0) {
    (void) alert(0,"gestls.c:setup_tls_ctx, error '%d', trouble with <%s>",
		   -phase,errmsg);
    phase=-1;
    }
  phase++;
  }
return ctx;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to catch ALRM signal.		*/
/*	(connection on socket program too long)	*/
/*						*/
/************************************************/
static void tlsconnectout(int sig)

{
switch (sig) {
  case SIGTERM	:
    quickout=true;
    break;
  default	:
    break;
  }
(void) siglongjmp(toolong,1);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to trace on a debug file the	*/
/*	string reported.			*/
/*	remove \r or \n character.		*/
/*						*/
/************************************************/
static void traceout(char *comment,char *str,int parnum)

{
if ((str!=(char *)0)&&(parnum>0)) {
  char *msg;
  char **split;

  msg=(char *)calloc(parnum+1,sizeof(char));
  (void) memcpy(msg,str,parnum);
  if ((split=eml_splitmsg(msg))!=(char **)0) {
    int i;

    for (i=0;split[i]!=(char *)0;i++) {
      (void) fprintf(stdout,"%s%s\n",comment,split[i]);
      (void) fflush(stdout);
      }
    split=rou_freestrlst(split);
    }
  (void) free(msg);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to display an TLS error	*/
/*						*/
/************************************************/
static void tlsshowerror(int dbg,SSL *ssl,int code,char *fmt,...)

{
int sslerr;
char *buff;
va_list args;
char info[300];

va_start(args,fmt);
buff="Premature EOF";
(void) vsnprintf(info,sizeof(info),fmt,args);
(void) tracing(TRALOG,dbg,info);
switch((sslerr=SSL_get_error(ssl,code))) {
  case SSL_ERROR_ZERO_RETURN		:
    buff="zero return";
    break;
  case SSL_ERROR_WANT_READ		:
    buff="read error";
    break;
  case SSL_ERROR_WANT_WRITE		:
    buff="write error";
    break;
  case SSL_ERROR_WANT_CONNECT		:
    buff="connect error";
    break;
  case SSL_ERROR_WANT_X509_LOOKUP	:
    buff="X509 lookup error";
    break;
  case SSL_ERROR_SYSCALL		:
    if (ERR_get_error()==0)
      buff="Premature EOF";
    else
      buff="syscall error";
    break;
  case SSL_ERROR_SSL			:
    buff=ERR_error_string(ERR_get_error(),(char *)0);
    break;
  }
(void) tracing(TRALOG,dbg,"SSL error=<%s>",buff);
va_end(args);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to build a ID string about	*/
/*	the ciphered connection under TLS	*/
/*						*/
/************************************************/
static char *tlsgetcipherid(SSL *ssl)

{
#define MXID 72

char *cipherid;
SSL_CIPHER *cipher;

cipherid=(char *)0;
if ((cipher=SSL_get_current_cipher(ssl))!=(SSL_CIPHER *)0) {
  char *version; 
  const char *name; 
  const char *verif;
  int bits;

  version=SSL_CIPHER_get_version(cipher);
  name=SSL_CIPHER_get_name(cipher);
  bits=SSL_CIPHER_get_bits(cipher,0);
  if (strcmp(version,"SSLv3")==0)
    version="TLSv1/SSLv3";
  cipherid=(char *)calloc(1,MXID);
  switch(SSL_get_verify_mode(ssl)) {
    case (SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE)	:
    case (SSL_VERIFY_NONE)				:
      switch(SSL_get_verify_result(ssl)) {
	case (X509_V_OK)	:
          verif="OK";
	  break;
	default			:
          verif="FAIL";
	  break;
	}
      break;
    default			:
      verif="NO";
      break;
    }
  (void) snprintf(cipherid,MXID,"(version=%s cipher=%s bits=%d verify=%s)",
				  version,name,bits,verif);
  }
return cipherid;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to accept a proposal to	*/
/*	encrypt a communication channel.	*/
/*	Return -1 if trouble, 0 otherwise.	*/
/*						*/
/************************************************/
static int tlsaccept(TFILE *tls)

{
#define	OPAC	"gestls.c:tlsaccept,"

int statut;
X509 *peer;
int tic;
int done;

statut=-1;
peer=(X509 *)0;
tic=0;
done=false;
if ((tls->bio=BIO_new_fd(tls->handle,BIO_NOCLOSE))==(BIO *)0) {
  (void) code_fault("%s Unable to get the BIO (error=<%s>)",
		    OPAC,strerror(errno));
  }
(void) SSL_set_bio(tls->ssl,tls->bio,tls->bio);
while (done==false) {
  int sslerr;

  statut=SSL_accept(tls->ssl);
  switch (sslerr=SSL_get_error(tls->ssl,statut)) {
    case SSL_ERROR_NONE	     :
      statut=0;
      if ((peer=SSL_get_peer_certificate(tls->ssl))!=(X509 *)0) {
        int s_x509;
  
        s_x509=SSL_get_verify_result(tls->ssl);
        if (SSL_get_verify_result(tls->ssl)!=X509_V_OK)
          statut=-1;
        (void) X509_free(peer);
        }
      done=true;
      break;
    case SSL_ERROR_WANT_READ :
      switch(tls_waitforchar(tls,1000)) {
        case -1		     :
          switch(errno) {	/*received a signal, lets see...*/
	    case EINTR	     :	/*could be a TERM signal	*/
	      break;	
	    default	     :	/*hummm real code fault report	*/
	      (void) code_fault("%s poll error '%s'",OPAC,strerror(errno));
              done=true;
	      statut=-1;
	      break;
	    }
	  break;
        case  0		     :	/*standard time out		*/
          tic++; 
          if (tic>MXTRY) {
            done=true;
            (void) alert(TRALOG,0,"%s, SSL_accept too long to establish",
				   OPAC);
	    statut=-1;
	    }
          break;
	default		     :
          break;
        }
      break;
#ifdef	SSL_ERROR_WANT_ACCEPT
    case SSL_ERROR_WANT_ACCEPT	:
#endif
    case SSL_ERROR_WANT_WRITE	:
      break;
    case SSL_ERROR_SYSCALL:	/*EOF received		*/
      if (ERR_get_error()==0)
	tls->goteof=true;		
      else
        (void) tlsshowerror(0,tls->ssl,statut,"%s SSL_SYSCALL pbs",OPAC);
      statut=-1;
      done=true;
      break;
    default				:
      (void) tlsshowerror(0,tls->ssl,statut,"%s SSL_accept fatal error "
					    "(sslerr='%d')",OPAC,sslerr);
      statut=-1;
      done=true;
      break; 
    }
  }
return statut;
}
/*

*/
/************************************************/
/*						*/
/*	Procedur to initiate a TLS connection	*/
/*	from the client side.			*/
/*	Return -1 if trouble, 0 otherwise	*/
/*						*/
/************************************************/
static int tlsconnect(TFILE *tls)

{
#define OPCO	"gestls.c:tlsconnect,"

int done;
int statut;
int tic;

done=false;
statut=-1;
tic=0;
if ((tls->bio=BIO_new_fd(tls->handle,BIO_NOCLOSE))==(BIO *)0) {
  (void) code_fault("%s Unable to get the BIO (error=<%s>)",
		    OPCO,strerror(errno));
  }
(void) SSL_set_bio(tls->ssl,tls->bio,tls->bio);
while (done==false) {
  statut=SSL_connect(tls->ssl);
  switch (SSL_get_error(tls->ssl,statut)) {
    case SSL_ERROR_NONE	     :
      done=true;
      statut=1;
      break;
    case SSL_ERROR_WANT_READ :
      switch(tls_waitforchar(tls,1000)) {
        case -1		     :
          switch(errno) {	/*received a signal, lets see...*/
	    case EINTR	     :	/*could be a TERM signal	*/
	      break;	
	    default	     :	/*hummm real code fault report	*/
	      (void) code_fault("%s poll error '%s'",OPCO,strerror(errno));
              done=true;
	      statut=-1;
	      break;
	    }
	  break;
        case  0		     :	/*standard time out		*/
          tic++; 
          if (tic>MXTRY) {
            done=true;
            (void) alert(TRALOG,0,"%s, SSL_connect too long to establish",OPCO);
	    statut=-1;
	    }
          break;
	default		     :
          break;
        }
      break;
    case SSL_ERROR_WANT_WRITE	:
    case SSL_ERROR_WANT_CONNECT	:
      break;
    default				:
      (void) tlsshowerror(0,tls->ssl,statut,"%s SSL_connect fatal error",OPCO);
      statut=-1;
      done=true;
      break;
    }
  }
return statut;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by a TLS	*/
/*	structure.				*/
/*						*/
/************************************************/
static TFILE *tlsclean(TFILE *tls)

{
if (tls!=(TFILE *)0) {
  tls->cipherid=rou_freestr(tls->cipherid);
  tls->carin=rou_freestr(tls->carin);
  (void) free(tls);
  tls=(TFILE *)0;
  }
return tls;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get a full line, if the	*/
/*	can't be read in one shot memory is	*/
/*	extended and a new probe is done	*/
/*						*/
/************************************************/
static char *tlsgetline(TFILE *tls,char *current)

{
#define	 B64K	65536

int gotit;

gotit=false;
while (gotit==false) {
  char buffer[512];

  (void) memset(buffer,'\000',sizeof(buffer));
  if (tls_fgets(buffer,sizeof(buffer),tls)==(char *)0) 
    break;
  current=rou_memstrcat(current,"%s",buffer);
  if (strlen(buffer)<sizeof(buffer)-1)
    break;
  if (strlen(current)>B64K) {
    (void) alert(0,"%s SMTP input line bigger than '%d'!",
		   "gestls.c:tlsgetline,",B64K);
    break;
    }
  }
return current;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to input character from the	*/
/*	socket.					*/
/*						*/
/************************************************/
int tls_getfeed(TLSFILE *tlsfile)

{
#define	OPFEED	"gestls.c:tlsgetfeed,"

TFILE *tls;
int limite;
char *ptr;
int cpt;

tls=(TFILE *)tlsfile;
limite=(MXBUF-1)-tls->incpt;
ptr=tls->carin+tls->incpt;
cpt=0;
errno=0;
switch (tls->tls) {
  case true	:
    cpt=SSL_read(tls->ssl,ptr,limite);
    break;
  default	:
    cpt=read(tls->handle,ptr,limite);
    break;
  }
switch (cpt) {
  case  0	:	/*no character available	*/
    switch (tls->tls) {
      case true			:
	{
	int big;

	switch ((big=SSL_get_error(tls->ssl,0))) {
	  case SSL_ERROR_SYSCALL:	/*EOF received		*/
    	    if (ERR_get_error()!=0)
	      (void) tracing(TRALOG,0,"%s wrong EOF",OPFEED);
	    else
              tls->goteof=true;		
	    break;
	  default		:
	    (void) tracing(TRALOG,0,"%s Unexpected SSL_read error='%d'",
				     OPFEED,SSL_get_error(tls->ssl,0));
	    break;
	  }
	(void) sleep(1);
	}
	break;
      default			:
        tls->goteof=true;		
        switch (errno) {
          case 0		:	/*EOF received		*/
	    break;
	  case EINTR		:	/*expected! error	*/
	  case EAGAIN		:
	  case ECONNRESET	:
	    break;
	  default		:
	    (void) tracing(TRALOG,0,"%s Unexpected read error=%d-><%s>",
				     OPFEED,errno,strerror(errno));
	    (void) tracing(TRALOG,0,"%s limite='%d', incpt='%d'",
				     OPFEED,limite,tls->incpt);
	    break;
	  }
        break;
      }
    break;
  case -1	:
    switch (tls->tls) {
      case true	:
	switch (SSL_get_error(tls->ssl,-1)) {
  	  case SSL_ERROR_WANT_READ	:
            (void) usleep(1000);	/*to ease up incoming charactere*/
	    break;
  	  case SSL_ERROR_SYSCALL	:
    	    if (ERR_get_error()==0)
	      tls->goteof=true;		
	    else
              (void) tlsshowerror(0,tls->ssl,cpt,"%s SSL_SYSCALL pbs",OPFEED);
	    break;
	  default	:
            (void) tlsshowerror(0,tls->ssl,cpt,"%s SSL_read problem",OPFEED);
	    break;
	  }
        break;
      default	:
        switch (errno) {
          case 0		:	/*success ???		*/
          case ESRCH		:	/*disconnect received	*/
	  case ECONNRESET	:	/*reset by peer		*/
	    break;
	  default		:
	    (void) tracing(TRALOG,0,"%s read error=%d-><%s>",
				     OPFEED,errno,strerror(errno));
	    tls->goteof=true;
	    break;
	  }
        break;
      }
    break;
  default	:
    tls->lastused=rou_systime();
    tls->incpt+=cpt;
    /*removing wrong char '\000' received	*/
    while ((ptr=memchr(tls->carin,'\000',tls->incpt))!=(void *)0) {
      *ptr='?';	/*replacing '\000' as '?'	*/
      }
    break;
  }
return (tls->goteof==false);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to write buffer on the proper	*/
/*	channel (either encrypted via TLS or	*/
/*	plain).					*/
/*						*/
/************************************************/
int tls_write(TLSFILE *tls,char *buffer,int parnum)

{
#define OPWTLS	"gestlst.c:tls_write,"

int sent;
TFILE *tfile;
int proceed;

sent=0;
tfile=(TFILE *)tls;
proceed=true;
if (tfile->goteof==true)
  proceed=false;
if (debug>4) {
  char *str;

  str="#";
  if (tfile->goteof==true)
    str="#eof#";
  (void) traceout(str,buffer,parnum);
  }
while (proceed==true) {
  switch (tfile->tls) {
    case  true	:
      sent=SSL_write(tfile->ssl,buffer,parnum);
      break;
    default	:
      sent=write(tfile->handle,buffer,parnum);
      break;
    }
  proceed=false;
  switch (sent) {
    case  -1  :
      switch (tfile->tls) {
        case  true	:
          switch (SSL_get_error(tfile->ssl,-1)) {
	    case SSL_ERROR_WANT_READ		:
	    case SSL_ERROR_WANT_WRITE		:
	      (void) usleep(10000);
	      proceed=true;	/*lets proceed again			*/
	      break;
	    default			:
              (void) tlsshowerror(0,tfile->ssl,sent,"%s SSL_write problem",OPWTLS);
	      break;
	    }
          break;
        default		:
	  switch (errno) {
	    case EAGAIN 	:	/*NONBLOCK device not ready yet	*/
	      (void) usleep(10000);
	      proceed=true;
	      break;
	    case ECONNRESET	:	/*remote just vanish		*/
	      break;			/*lets get out with the error	*/
	    default		:
              (void) alert(0,"%s Write problem, (error=<%s>)",
		              OPWTLS,strerror(errno));
	      break;
	    }
          break;
        }
      break;
    case   0  :		/*no character written	*/
      tfile->goteof=true;		
      switch (tfile->tls) {
        case true			:
	  {
	  switch (SSL_get_error(tfile->ssl,0)) {
	    case SSL_ERROR_SYSCALL:	/*EOF received		*/
    	      if (ERR_get_error()!=0)
	        (void) tracing(TRALOG,0,"%s wrong EOF",OPFEED);
	      break;
	    default		:
	      (void) tracing(TRALOG,0,"%s Unexpected SSL_write error='%d'",
				       OPWTLS,SSL_get_error(tfile->ssl,0));
	      break;
	    }
	  (void) sleep(1);
	  }
	  break;
        default			:
          switch (errno) {
            case 0		:	/*EOF received		*/
	      break;
	    case EINTR		:	/*expected! error	*/
	    case EAGAIN		:
	    case ECONNRESET	:
	      break;
	    default		:
	      (void) tracing(TRALOG,0,"%s Unexpected write error=<%s>",
				       OPWTLS,strerror(errno));
	      break;
	    }
          break;
        }
      break;
    default	:
      if ((parnum-sent)>0) {
	parnum-=sent;
	(void) memmove(buffer,buffer+sent,parnum);
	proceed=true;
	}
      tfile->lastused=rou_systime();
      break;
    }
  }
return sent;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait a connection from	*/
/*	remote and assign the structure to	*/
/*	mamange a crypted connexion (TLS)	*/
/*						*/
/************************************************/
TLSFILE *tls_accept(LSTTYP *lst)

{
TLSFILE *tls;
int handle;

tls=(TLSFILE *)0;
if ((handle=dev_acceptfile(lst))>=0) {
  TFILE *chantls; 

  chantls=(TFILE *)calloc(1,sizeof(TFILE));
  chantls->carin=calloc(1,MXBUF);
  chantls->handle=handle;
  tls=(TLSFILE *)chantls;
  }
return tls;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to put the link in encrypted	*/
/*	mode.					*/
/*	Return -1 if trouble, 0 otherwise	*/
/*						*/
/************************************************/
int tls_gotls(TLSFILE *tlsfile,int accepting)

{
#define	OPST	"gesttls.c:tls_gotls,"
#define TLSWAIT	120

int statut;
TFILE *tls;

statut=-1;
tls=(TFILE *)tlsfile;
if (tls->tls==true) {
  (void) tracing(TRALOG,2,"%s TLS link already crypted",OPST);
  }
else {
  (void) SSL_library_init();
  (void) SSL_load_error_strings();
  (void) ERR_clear_error();
  if ((tls->ctx=setup_tls_ctx())!=(SSL_CTX *)0) {
    if ((tls->ssl=SSL_new(tls->ctx))!=(SSL *)0) {
      switch (accepting)	{
        case false	:	/*client initiate connection	*/
          statut=tlsconnect(tls);
	  break;
        case true	:	/*server accept connection	*/
	default		:	/*no break			*/
          statut=tlsaccept(tls);
	  break;
        }
      if (statut<0) {
	if (tls->goteof==false)
          (void) alert(0,"%s Unable to assign fd to SSL",OPST);
        (void) SSL_CTX_free(tls->ctx);
        tls->ctx=(SSL_CTX *)0;
        (void) ERR_free_strings();
	(void) SSL_free(tls->ssl);
	tls->ssl=(SSL *)0;
        }
      else {
	tls->cipherid=tlsgetcipherid(tls->ssl);
        tls->tls=true;
	}
      }
    }
  }
return statut;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return the cipher infos	*/
/*	extracted from the TLS connection	*/
/*						*/
/************************************************/
char *tls_getcipher(TLSFILE *tlsfile)

{
char *cipherid;
TFILE *tls;

cipherid=(char *)0;
tls=(TFILE *)tlsfile;
if ((tls->tls==true)&&(tls->cipherid!=(char *)0)) {
  cipherid=strdup(tls->cipherid);
  }
return cipherid;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to terminate a TLS link.	*/
/*						*/
/************************************************/
int tls_close(TLSFILE *tlsfile)

{
#define 	APCO	"gestls.c:tls_close,"
TFILE *tls;
int status;

tls=(TFILE *)tlsfile;
if (tls->tls==true) {
  (void) SSL_set_shutdown(tls->ssl,SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
  switch((status=SSL_shutdown(tls->ssl))) {
    case -1	:
      (void) tlsshowerror(0,tls->ssl,status,"%s SSL_shutdown fatal error",APCO);
      break;
    case  0	:
      if ((status=SSL_shutdown(tls->ssl))!=1) 
        (void) tlsshowerror(0,tls->ssl,status,"%s still in HALF shutdown",APCO);
      break;
    case  1	:		/*good default				*/
      break;
    default	:
      (void) code_fault("%s unexpected shutdown (statut='%d')",APCO,status);
      break;
    }
  (void) SSL_free(tls->ssl);
  (void) SSL_CTX_free(tls->ctx);
  tls->ctx=(SSL_CTX *)0;
  }
if ((status=shutdown(tls->handle,SHUT_RDWR))<0) {
  switch (errno) {
    case ENOTCONN	:	/*already disconnect by other side	*/
      break;
    default		:
      (void) alert(0,"%s unable to shutdown (errno=<%s>)",APCO,strerror(errno));
      break;
    }
  }
(void) close(tls->handle);
tls=tlsclean(tls);
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Routine to send characater on socket,	*/
/*	either in plain text or on TLS 		*/
/*	encryption.				*/
/*	This entry always add a '\n\r' to the	*/
/*	string sent on the main channel.	*/
/*						*/
/************************************************/
int tls_printf(TLSFILE *tlsfile,const char *fmt,...)

{
va_list args;
char *local;
int parnum;

va_start(args,fmt);
local=rou_dynavsprintf(fmt,args);
parnum=strlen(local);
(void) tls_write(tlsfile,local,parnum);
(void) free(local);
va_end(args);
return parnum;
}
/*

*/
/************************************************/
/*						*/
/*	Routine to read from a socket either	*/
/*	in plain text or in TSL encryption	*/
/*						*/
/************************************************/
char *tls_fgets(char *s,int size,TLSFILE *tlsfile)

{
#define OPRTLS	"unitls.c:tls_fgets,"

TFILE *tls;
char *str;
int limite;

tls=(TFILE *)tlsfile;
str=(char *)0;
(void) memset(s,'\000',size);
if (tls->incpt>0) {
  char *stop;

  limite=size-1;
  if (limite>tls->incpt)
    limite=tls->incpt;
  (void) memmove(s,tls->carin,limite);
  stop=strchr(s,'\n');
  if (stop==(char *)0) {
    stop=strchr(s,'\r');
    }
  if (stop!=(char *)0) {
    stop++;
    *stop='\000';
    }
  else {
    if (tls->incpt<size-1) {
      /*lets wait for full line	*/
      *s='\000';
      }
    }
  limite=strlen(s);
  if (limite>0) {
    tls->incpt-=limite;
    (void) memmove(tls->carin,tls->carin+limite,tls->incpt);
    str=s;	/*line is NOT empty	*/
    }
  tls->carin[tls->incpt]='\000';
  }
if ((debug>0)&&(str!=(char *)0)) {
  (void) traceout("",str,strlen(str));
  }
return str;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get a name related to an	*/
/*	TLS connection.				*/
/*	Return a newly allocated memory which	*/
/*	contents is the needed information.	*/
/*						*/
/************************************************/
char *tls_getaddrname(TLSFILE *tlsfile,PEERTYP peer,char *unknown)

{
TFILE *tls;

tls=(TFILE *)tlsfile;
return dev_getaddrname(tls->handle,peer,unknown);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to open a TCP connexion to a	*/
/*	daemon on a specific channel		*/
/*						*/
/************************************************/
TLSFILE *tls_opentcp(char *demon,int port,int swait)

{
#define	OPTC "gestsl.c:tls_opentcp,"

TFILE *tls;
int handle;
struct hostent *h;
struct sockaddr_in sosin;
void (*prvalrm)(int);
void (*prvterm)(int);

tls=(TFILE *)0;
if ((h=(struct hostent *)gethostbyname(demon))==(struct hostent *)0) {
  (void) alert(2,"%s Unable to get '%s' host IP (error='%s')",
		 OPTC,demon,strerror(errno));
  return (TLSFILE *)tls;
  }
if ((handle=socket(AF_INET,SOCK_STREAM,0))<0) {
  (void) code_fault("%s Unable to open socket (error='%s')",OPTC,strerror(errno));
  }
sosin.sin_port=htons(port);
sosin.sin_family=h->h_addrtype;
(void) memcpy(&sosin.sin_addr.s_addr,h->h_addr,h->h_length);
prvalrm=signal(SIGALRM,SIG_IGN);
prvterm=signal(SIGTERM,SIG_IGN);
if (sigsetjmp(toolong,true)==0) {
  (void) signal(SIGALRM,tlsconnectout);
  (void) signal(SIGTERM,tlsconnectout);
  (void) alarm(swait);
  if (connect(handle,(struct sockaddr *)&sosin,sizeof(sosin))==0) {
    int flags;

    tls=(TFILE *)calloc(1,sizeof(TFILE));
    tls->carin=calloc(1,MXBUF);
    if ((flags=fcntl(handle,F_GETFL,0))<0)
      (void) code_fault("%s Unable to get socket descripteur (error='%s')",
		        OPTC,strerror(errno));
    if ((flags=fcntl(handle,F_SETFL,flags|O_NONBLOCK|O_ASYNC))<0)
       (void) code_fault("%s Unable to set socket descripteur (error='%s')",
		         OPTC,strerror(errno));
    tls->handle=handle;
    }
  else {
    (void) alert(1,"%s '%s' (port %d) unable to make connection (error=<%s>)",
		   OPTC,demon,port,strerror(errno));
    (void) close(handle);
    }
  }
else {
  (void) alert(1,"%s '%s/%d' unable to get answer from "
		 "remote (error=<Timeout>)",OPTC,demon,port);
  (void) close(handle);
  }
(void) alarm(0);
(void) signal(SIGALRM,prvalrm);
(void) signal(SIGTERM,prvterm);
return (TLSFILE *)tls;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for characters	*/
/*	available on a TLS channel.		*/
/*						*/
/************************************************/
int tls_waitforchar(TLSFILE *tlsfile,u_int millisec)

{
int status;
TFILE *tls;
struct pollfd polling[1];

status=-1;
tls=(TFILE *)tlsfile;
if ((tls==(TFILE *)0)||(tls->handle<0)) {
  errno=EBADF;
  return status;
  }
switch (tls->tls) {
  case true	:
    polling[0].fd=SSL_get_fd(tls->ssl);
    break;
  default	:
    polling[0].fd=tls->handle;
    break;
  }
polling[0].events=POLLIN|POLLPRI;
polling[0].revents=(short)0;
status=poll(polling,1,millisec);
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for data coming from	*/
/*	remote SMTP client, same time we detecte*/
/*	if Parent process requested a quickout	*/
/*	or just vanished.			*/
/*	such case we return with a null pointer	*/
/*	Assign a dynamic memory to store input	*/
/*	SMTP line.				*/
/*						*/
/************************************************/
char *tls_waitforaction(int secwait,TLSFILE *tls)

{
#define	OPWA	"gestls.c:t,tls_waitforaction"

char *info;
int i;

info=(char *)0;
info=tlsgetline(tls,info);
for (i=0;i<secwait;i++) {
  if (info!=(char *)0) {
    register char *ptr;

    ptr=info+strlen(info)-1;
    if ((*ptr=='\n')||(*ptr=='\r')) {
      break;			/*we got one line OK		*/
      }
    }
  switch(tls_waitforchar(tls,1000)) {
    case -1		:
      switch(errno) {
	case EINTR	:	/*received a signal, lets see...*/
	  break;		/*could be a TERM signal	*/
	default		:	/*hummm real code fault report	*/
	  (void) code_fault("%s poll error '%s'",OPWA,strerror(errno));
	  break;
	}
      break;
    case  0		:	/*standard time out		*/
      break;
    default		:
      if (tls_getfeed(tls)==true)
        info=tlsgetline(tls,info);
      break;
    }
  if ((tls_iseof(tls)==true) ||
      (quickout==true)||
      (rou_checkprocess(getppid())==false)) {
    info=rou_freestr(info);
    break;			/*going out fast!		*/
    }
  }
return info;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to report the tls EOF status	*/
/*						*/
/************************************************/
int tls_iseof(TLSFILE *tlsfile)

{
TFILE *tls;

tls=(TFILE *)tlsfile;
return tls->goteof;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to report the number of second*/
/*	The TLS connection is IDLE.		*/
/*						*/
/************************************************/
u_long tls_idle(TLSFILE *tlsfile)

{
TFILE *tls;

tls=(TFILE *)tlsfile;
return (u_long)(rou_systime()-tls->lastused);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to convert an ASCII B64	*/
/*	Sequence to a plain ASCII sequence.	*/
/*						*/
/************************************************/
char **tls_getb64(char **lst,const char *b64)

{
BIO *mbio;
BIO *bbio;
int outlen;
char *decode;

decode=(char *)calloc(1,strlen(b64)+3);
mbio=BIO_new(BIO_s_mem());
bbio=BIO_new(BIO_f_base64());
BIO_set_flags(bbio,BIO_FLAGS_BASE64_NO_NL);
bbio=BIO_push(bbio,mbio);
(void) BIO_write(mbio,b64,strlen(b64));
(void) BIO_flush(mbio);
if ((outlen=BIO_read(bbio,decode,strlen(b64)))<0)
  (void) tracing(TRALOG,0,"gestls.c:tls_getb64, BIO_read error=<%s>\n",
			   strerror(errno));
else {
  int i;

  for (i=0;i<outlen;i++) {
    char *ptr;

    ptr=decode+i;
    lst=(char **)rou_addveclst((void **)lst,(void *)(strdup(ptr)));    
    i+=strlen(ptr);
    }
  }
(void) BIO_free_all(bbio);
(void) free(decode);
return lst;
}
