/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	LVLCLM:					*/
/*	Take Care of all "CLAMAV"  anti-virus	*/
/*	scanner access.				*/
/*						*/
/************************************************/
#include	<string.h>
#include	<stdio.h>
#include	<time.h>
#include	<setjmp.h>
#include	<errno.h>
#include	<signal.h>
#include	<syslog.h>
#include	<stdlib.h>
#include        <execinfo.h>
#include	"dbgmem.h"
#include	"subrou.h"
#include	"unieml.h"
#include	"gestls.h"
#include	"gesque.h"
#include	"lvlcla.h"

/*how long to wait for Virus scanner		*/
#define	WAITOPEN	10
/*how long to check about reloading library	*/
#define	MXRETRY	       600

typedef	struct	{
	time_t last;		/*last load	*/
	int retry;		/*retry time;	*/
	int port;		/*scanner port	*/
	char *hostname;		/*scanner names	*/
	char *version;		/*version used	*/
	char *virname;		/*last virus	*/
	}CLAMFILE;
/*

*/
/************************************************/
/*						*/
/*	Routine to check the result from the	*/
/*	virus scanner				*/
/*						*/
/************************************************/
static CLMCHKTYP lookatscanres(char *result,CLAMFILE *clamfile)

{
#define	OPLO	"lvlclm.c:lookatscanres,"

#define	LF	"\r"

static VOCTYP vocckr[]={
	{1,"ERROR",(void *)0},
	{2,"FOUND",(void *)0},
	{3,"OK",(void *)0},
	{0,(char *)0,(void *)0}
	};

CLMCHKTYP status;
char *ptr;
int phase;
int proceed;

status=cla_nocheck;
ptr=(char *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*extracting status	*/
      if ((ptr=strrchr(result,' '))==(char *)0) {
        (void) alert(0,"%s no space found within <%s> (bug?)",OPLO,result);
	phase=999;
	}
      break;
    case 1	:	/*extracting status	*/
      if (ptr!=(char *)0) {	/*alway true	*/
	VOCTYP *voc;

	if ((voc=rou_getvoca(vocckr,ptr+1))!=(VOCTYP *)0) {
  	  switch (voc->code) {
	    case 1  :	/*Error!		*/
              (void) alert(0,"%s Error found within <%s> (bug?)",OPLO,result);
	      break;
	    case 2  :	/*Virus Found		*/
	      status=cla_virus;
	      break;
	    case 3  :	/*everything OK		*/
	      status=cla_clean;
	      break;
    	    default :	/*big trouble		*/
              (void) alert(0,"%s Unexpected scanner returned result <%s> (bug?)",OPLO,result);
	      break;
	    }
	  }
	}
      break;
    case 2	:	/*extracting virus Name	*/
      clamfile->virname=rou_freestr(clamfile->virname);
      switch (status) {
	case cla_virus	:
	  *ptr='\000';
          if ((ptr=strrchr(result,' '))==(char *)0) {
            (void) alert(0,"%s No virus name found within <%s> (bug?)",OPLO,result);
	    ptr=" Unnamed Virus";
	    }
	  ptr++;
	  clamfile->virname=strdup(ptr);	
	  break;
	case cla_clean	:	/*nothing to do	*/
	case cla_nocheck:
	  break;
	}
      break;
    default	:	/*SAFE guard		*/
      proceed=false;
      break;
    }
  phase++;
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	procedure to send a file on specific	*/
/*	clamav scanner port.			*/
/*						*/
/************************************************/
static int streaming(char *hostname,int port,const char *filename)

{
#define	OPST	"lvlclm.c:streaming,"

int status;
FILE *fichier;
TLSFILE *tlsfile;
int phase;
int proceed;

status=false;
fichier=(FILE *)0;
tlsfile=(TLSFILE *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*opening file		*/
      if ((fichier=fopen(filename,"r"))==(FILE *)0) {
        (void) alert(0,"%s Unable to open file <%s> (error=<%s>)",
			OPST,filename,strerror(errno));
	proceed=false;
	}
      break;
    case 1	:	/*opening channel	*/
      if ((tlsfile=tls_opentcp(hostname,port,WAITOPEN))==(TLSFILE *)0) {
        (void) alert(0,"%s Unable to access <%s> port '%d' within '%d' sec (config?)",
			OPST,hostname,port,WAITOPEN);
	phase=999;	/*trouble trouble	*/
	}
      break;
    case 2	:	/*sending file		*/
      if (fichier!=(FILE *)0) {	/*always true	*/
	char buffer[2048];

        while (fgets(buffer,sizeof(buffer),fichier)!=(char *)0) {
          (void) tls_printf(tlsfile,"%s",buffer);
	  }
        }
      break;
    case 3	:	/*closing channel	*/
      (void) tls_close(tlsfile);
      status=true;
      break;
    default	:	/*SAFE Guard		*/
      (void) fclose(fichier);
      proceed=false;
      break;
    }
  phase++;  
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	procedure to send ONE string to a remote*/
/*	TCP server.				*/
/*						*/
/************************************************/
static char *sendoneshot(char *hostname,int port,int mxwait,const char *fmt,...)

{
#define	OPSE		"lvlclm.c:sendoneshot,"

register int dummycnt;
char *buffer;
va_list args;
TLSFILE *tlsfile;
int phase;
int proceed;

dummycnt=0;
buffer=(char *)0;
va_start(args,fmt);
tlsfile=(TLSFILE *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*Opening the channel	*/
      if ((tlsfile=tls_opentcp(hostname,port,WAITOPEN))==(TLSFILE *)0) {
        (void) alert(3,"%s Unable to access <%s> port '%d' within '%d' sec (config?)",
			    OPSE,hostname,port,WAITOPEN);
	phase=999;	/*trouble trouble	*/
        }
      break;
    case 1	:	/*sending the string	*/
      dummycnt=vasprintf(&buffer,fmt,args);
      (void) tls_printf(tlsfile,"%s%s",buffer,LF);
      (void) free(buffer);
      break;
    case 2	:	/*expecting answer	*/
      if ((buffer=tls_waitforaction(mxwait,tlsfile))==(char *)0) {
	char *local;
	va_list argssup;

	va_start(argssup,fmt);
        dummycnt=vasprintf(&local,fmt,argssup);
        (void) alert(0,"%s No answer to <%s> from scanner within '%d' sec",
			OPSE,local,mxwait);
	(void) free(local);
	va_end(argssup);
	}
      break;
    case 3	:	/*closing channel	*/
      (void) tls_close(tlsfile);
      break;
    default	:	/*SAFE guard		*/
      proceed=false;
      break;
    }
  phase++;
  }
va_end(args);
return buffer;
}
/*

*/
/************************************************/
/*						*/
/*	Routine to check if a new virus tree is	*/
/*	needed to be loaded			*/
/*						*/
/************************************************/
int cla_reload(CLAMTYP *clamav)

{
#define	OPRE	"lvlclm.c:clm_reload,"

#define	CLMRELOAD	"RELOAD"
#define	CLMRELOADING	"RELOADING"
#define	CLMVERSION	"VERSION"
#define	WRELOAD		15

int status;

status=false;
if (clamav!=(CLAMTYP *)0) {
  CLAMFILE *cf;
  time_t curtime;

  cf=(CLAMFILE *)clamav;
  curtime=time((time_t *)0);
  status=((cf->last+cf->retry)>=curtime);
  if (status==false) {
    char *buffer;
    int phase;
    int proceed;

    buffer=(char *)0;
    phase=0;
    proceed=true;
    while (proceed==true) {
      switch (phase) {
	case 0	:	/*Sending reload request*/
	  if ((buffer=sendoneshot(cf->hostname,cf->port,WRELOAD,"%s",CLMRELOAD))==(char *)0)
	    phase=999;	/*trouble trouble	*/
	  break; 
	case 1	:	/*checking reload OK	*/
	  buffer=que_cleanstring(buffer);
	  if (strcmp(buffer,CLMRELOADING)!=0) {
            (void) alert(0,"%s answer <%s> is not expected one <%s>",
			    OPRE,buffer,CLMRELOADING);
	    phase=999;	/*trouble trouble	*/
	    }
	  (void) free(buffer);
	  break;
	case 2	:	/*asking version #	*/
	  cf->version=rou_freestr(cf->version);
	  if ((buffer=sendoneshot(cf->hostname,cf->port,WRELOAD,"%s",CLMVERSION))==(char *)0)
	    phase=999;	/*trouble trouble	*/
	  break;
	case 3	:	/*getting version #	*/
	  buffer=que_cleanstring(buffer);
	  cf->version=buffer;
	  status=true;	
			/*NO BREAK		*/
	default	:	/*SAFE Guard		*/
	  proceed=false;
	  break;
	}
      phase++;
      }
    cf->last=curtime;
    }
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to prepare a virus scan list	*/
/*						*/
/************************************************/
CLAMTYP *cla_openscanner(char *hostname,int port)

{
#define	OPSCAN		"lvlclm.c:clm_openscanner,"

#define	HOSTCAN		"localhost.localdomain"
#define	PORTSCAN	3310
#define	WAITSCAN	15	/*15 sec waiting*/
#define	CLMSESSION	"SESSION"

CLAMFILE *clamfile;
char *buffer;
int phase;
int proceed;

clamfile=(CLAMFILE *)0;
buffer=(char *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*setting default value	*/
      if (hostname==(char *)0)
	hostname=HOSTCAN;
      if (port==0)
	port=PORTSCAN;
      clamfile=(CLAMFILE *)calloc(1,sizeof(CLAMFILE));
      clamfile->retry=MXRETRY;
      clamfile->hostname=strdup(hostname);
      clamfile->port=port;
      break;
    case 1	:	/*checking reload	*/
      if (cla_reload((CLAMTYP *)clamfile)==false) {
        (void) alert(3,"%s Unable to reload data-base canceling access",OPSCAN);
	}
      break;
    default	:	/*SAFE Guard		*/
      proceed=false;
      break;
    }
  phase++;
  }
return (CLAMTYP *)clamfile;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free a virus scan list	*/
/*						*/
/************************************************/
CLAMTYP *cla_closescanner(CLAMTYP *clamav)

{
#define	CLMEND		"END"

if (clamav!=(CLAMTYP *)0) {
  CLAMFILE *clamfile;

  clamfile=(CLAMFILE *)clamav;
  clamfile->virname=rou_freestr(clamfile->virname);
  clamfile->version=rou_freestr(clamfile->version);
  clamfile->hostname=rou_freestr(clamfile->hostname);
  (void) free(clamfile);
  clamfile=(CLAMFILE *)0;
  clamav=(CLAMTYP *)clamfile;
  }
return clamav;
}
/*

*/
/************************************************/
/*						*/
/*	Routine to check if a virus is found	*/
/*	within the E-mail contents.		*/
/*	If visur discovered, return the virus	*/
/*	name.					*/
/*						*/
/************************************************/
CLMCHKTYP cla_checkvirus(CLAMTYP **clamav,char *filename,const char **virname)

{
#define	OPCH		"lvlclm.c:clm_checkvirus,"
#define	CLMWSCAN	30	/*wait 30 sec 	*/
#define	CLMSTREAM	"STREAM"/*STREAM file	*/
#define	CLMPORT		"PORT"	/*PORT NUM	*/

CLMCHKTYP status;

status=cla_nocheck;
*virname=(char *)0;
if (*clamav!=(CLAMTYP *)0) {
  CLAMFILE *cf;
  TLSFILE *tlsfile;
  char *buffer;
  char *strport;
  int streamport;
  int phase;
  int proceed;

  cf=(CLAMFILE *)*clamav;
  tlsfile=(TLSFILE *)0;
  buffer=(char *)0;
  strport=(char *)0;
  streamport=0;
  phase=0;
  proceed=true;
  while (proceed==true) {
    switch (phase) {
      case 0	:	/*opening channel	*/
        if ((tlsfile=tls_opentcp(cf->hostname,cf->port,WAITOPEN))==(TLSFILE *)0) {
          (void) alert(3,"%s Unable to access <%s> port '%d' within '%d' sec (config?)",
			  OPCH,cf->hostname,cf->port,WAITOPEN);
	  proceed=false;/*trouble trouble	*/
          }
        break;
      case 1	:	/*waiting answer	*/
        (void) tls_printf(tlsfile,"%s%s",CLMSTREAM,LF);
	break;
      case 2	:	/*check the answer	*/
        if ((buffer=tls_waitforaction(WAITOPEN,tlsfile))==(char *)0) {
          (void) alert(0,"%s No answer to <%s> from scanner within '%d' sec",
			  OPCH,CLMSTREAM,WAITOPEN);
          phase=999;	/*trouble trouble	*/
          }
	break;
      case 3	:	/*checking the answer	*/
	buffer=que_cleanstring(buffer);
        strport=calloc(strlen(buffer),sizeof(char));
	if (sscanf(buffer,"%s %d",strport,&streamport)!=2) {
          (void) alert(0,"%s answer <%s> format incorrect (clamav bug?)",OPCH,CLMSTREAM);
	  (void) free(strport);
          phase=999;	/*trouble trouble	*/
	  }
        (void) free(buffer);
	break;
      case 4	:	/*answer OK?		*/
	if (strcmp(strport,CLMPORT)!=0) {
          (void) alert(0,"%s Unexpected CLMPORT <%s> (clamav bug?)",OPCH,strport);
          phase=999;	/*trouble trouble	*/
	  }
	(void) free(strport);
	break;
      case 5	:	/*streaming file 	*/
	if (streaming(cf->hostname,streamport,filename)==false) {
          (void) alert(0,"%s Aborting virus detection",OPCH);
          phase=999;	/*trouble trouble	*/
	  }
	break;
      case 6	:	/*expecting answer	*/
        if ((buffer=tls_waitforaction(CLMWSCAN,tlsfile))==(char *)0) {
          (void) alert(0,"%s No answer scanning file <%s> within '%d' sec",
			  OPCH,filename,CLMWSCAN);
	  (void) tls_close(tlsfile);
          phase=999;	/*trouble trouble	*/
	  }
	break;
      case 7	:	/*checking the answer	*/
	buffer=que_cleanstring(buffer);
	switch (status=lookatscanres(buffer,cf)) {
	  case cla_clean	:
	    break;	/*nothing to do		*/
	  case cla_virus	:
	    *virname=cf->virname;
	    break;	/*get virusname		*/
	  case cla_nocheck	:
	    break;	/*nothing to do		*/
	  }
        (void) free(buffer);
	break;
      default	:	/*SAFE Guard		*/
        (void) tls_close(tlsfile);	
        tlsfile=(TLSFILE *)0;
	proceed=false;
	break;
      }
    phase++;
    }
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Routine to return the clamd version	*/
/*	and the last update date.		*/
/*						*/
/************************************************/
char *cla_getversdate(CLAMTYP *clamav)

{
register int dummycnt;
char *versdate;
CLAMFILE *clamfile;

versdate=(char *)0;
clamfile=(CLAMFILE *)clamav;
dummycnt=asprintf(&versdate,"%d %s",CMDOK,clamfile->version);
return versdate;
}
