/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	MODCHK:					*/
/*	Module level routines to handle		*/
/*	E-mail RCPT request by checking	with	*/
/*	local sender and remote Mentor.		*/
/*						*/
/************************************************/
#include	<stdlib.h>
#include	<unistd.h>
#include	<malloc.h>
#include	<syslog.h>
#include	<string.h>
#include	<math.h>
#include	<errno.h>
#include	<string.h>
#include	<signal.h>
#include	"subrou.h"
#include	"unieml.h"
#include	"unidns.h"
#include	"unicac.h"
#include	"gesque.h"
#include	"lvluid.h"
#include	"mngusr.h"
#include	"modchk.h"
typedef	enum	{	/*message type		*/
	msg_ckrcpt,	/*check RCPT info	*/
	msg_ckrsnd,	/*RCPT info (via sender)*/
	msg_ckrrly,	/*RCPT info (via relay)	*/
	msg_ckrmnt,	/*RCPT info (via mentor)*/
	msg_advice,	/*Requesting Mentor adv	*/
	msg_uniqid,	/*sending Uniqid	*/
	msg_chkid,	/*Check user ID		*/
	msg_refresh,	/*send refresh status	*/
	msg_sent,	/*sent acknowledge	*/
	msg_goodmail,	/*got a goodmail advice	*/
	msg_unknown	/*unknown message	*/
	}MENUM;

typedef struct {	/*adv dispatcher struct	*/
	pid_t chkpid;	/*sending channel pid	*/
	char *msg;	/*message to send	*/
	}DISTYP;

typedef struct	{	/*checker list		*/
	char *chkname;
	char *lst;
	}SNDTYP;
/*

*/
/************************************************/
/*						*/
/*	Routine to check spamlevel according	*/
/*	limit set a the domain level.		*/
/*						*/
/************************************************/
static void checkspam(ADVIMSG *advimsg,DOMCHK *dom)

{
if (dom!=(DOMCHK *)0) {
  int spamdflt;
  int i;

  spamdflt=false;
  if (advimsg->relayable==false)
    if (advimsg->spmlvl>=advimsg->spmlim)
      spamdflt=true;
  for (i=0;dom->rcpts[i]!=(RCPADV *)0;i++) {
    int itsspam;
    char *domain;
    DFSTYP *df;

    itsspam=spamdflt;
    domain=eml_strdomain(dom->rcpts[i]->email);
    dom->rcpts[i]->spamlim=advimsg->spmlim;
    if ((df=eml_locatedfs(dom->dfs,domain))!=(DFSTYP *)0) {
      if (df->spamlim!=NAN) {
        dom->rcpts[i]->spamlim=df->spamlim;
	itsspam=(advimsg->spmlvl>=df->spamlim);
	}
      }
    dom->rcpts[i]->spam=itsspam;
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	routine to locate one SNDTYP among an	*/
/*	SNDTYP list.				*/
/*						*/
/************************************************/
static SNDTYP *findsentlst(SNDTYP **lst,char *chkname)

{
SNDTYP *found;

found=(SNDTYP *)0;
if (lst!=(SNDTYP **)0) {
  int i;

  for (i=0;lst[i]!=(SNDTYP *)0;i++) {
    if (strcmp(lst[i]->chkname,chkname)==0) {
      found=lst[i];
      break;
      }
    }
  }
return found;
}
/*

*/
/************************************************/
/*						*/
/*	routine to extract the message code	*/
/*						*/
/************************************************/
static MENUM getmsgcode(char *message)

{
static	struct	{
	MENUM code;	/*message CODE		*/
	char *key;
	}msgcode[]={
		{msg_ckrcpt,CKRCPT},
		{msg_ckrsnd,CKRSND},
		{msg_ckrrly,CKRRLY},
		{msg_ckrmnt,CKRMNT},
		{msg_advice,ADVICE},
		{msg_uniqid,UNIQID},
		{msg_chkid,CHKID},
		{msg_goodmail,GMAIL},
		{msg_refresh,REFRESH},
		{msg_sent,SENT},
		{msg_unknown,(char *)0}
		};

MENUM code;
int i;

code=msg_unknown;
for (i=0;msgcode[i].key!=(char *)0;i++) {
  if (strstr(message,msgcode[i].key)==message) {
    (void) strcpy(message,message+strlen(msgcode[i].key));
    code=msgcode[i].code;
    break;
    }
  }
return code;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to locate a dispatch channel	*/
/*	previously created to request advice.	*/
/*						*/
/************************************************/
static DISTYP *getdispatch(DISTYP **dlst,pid_t chkpid)

{
DISTYP *found;

found=(DISTYP *)0;
if (dlst!=(DISTYP **)0) {
  int i;

  for (i=0;dlst[i]!=(DISTYP *)0;i++) {
    if (dlst[i]->chkpid==chkpid) {
      found=dlst[i];
      break;
      }
    }
  }
return found;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait response from remote	*/
/*	checker.				*/
/*						*/
/************************************************/
static char *gettlsline(char *line,int taille,CTTYP *contact)

{
char *ptr;

line[0]='\000';
if ((ptr=tls_fgets(line,taille,contact->tls))==(char *)0) {
  switch(tls_waitforchar(contact->tls,1000)) {
    case -1		:
      switch(errno) {
	case EINTR	:	/*received a signal, lets see...*/
	  break;		/*must be an IPC messages	*/
	default		:	/*hummm real code fault report	*/
	  (void) code_fault("%s select error '%s'",
			    "modchk.c:gettlsline",strerror(errno));
	  break;
	}
      break;
    case  0		:
     				/*standard time out		*/
      break;
    default		:
      if (tls_getfeed(contact->tls)==true)
        ptr=tls_fgets(line,taille,contact->tls);
      break;
    }
  }
if (ptr!=(char *)0) {
  (void) que_cleanstring(line);
  }
return ptr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to relay an TCP message to	*/
/*	to the IPC channel.			*/
/*						*/
/************************************************/
static void relaytoipc(BOXTYP *boxe,char **msg)

{
#define	OPRL	"modchk.c:relaytoipc,"

char *msgref;
int phase;
int proceed;

msgref=0;
phase=0;
proceed=true;
while (proceed==true) {
  switch(phase) {
    case 0	:
      if (rou_vecsize((void **)msg)<3) {
	int num;
	int i;

	(void) alert(0,"%s message too small",OPRL);
	num=rou_vecsize((void **)msg);
	for (i=0;i<num;i++)
	  (void) alert(0,"%s message[%d]=<%s>",OPRL,i,msg[i]);
	phase=999;
	}
      break;
    case 1	: 
      if (strstr(msg[0],ANSWID)!=msg[0]) {
	(void) alert(0,"%s Wrong message ID <%s>",msg[0]);
	phase=999;
	}
      break;
    case 2	: 
      if (msg!=(char **)0) {
        int i;	
        char *ipcmsg;

	ipcmsg=(char *)0;
	(void) strcpy(msg[0],msg[0]+strlen(ANSWID));
        for (i=1;msg[i]!=(char *)0;i++) {
          if ((msg[i][0]=='.')&&(msg[i][1]=='\000'))
	    break;
	  ipcmsg=rou_memstrcat(ipcmsg,"%s\n",msg[i]);
          }
        if (ipc_postreply(boxe,msg[0],ipcmsg)==0) {
	  (void) alert(0,"%s Unable to post, msgref=<%s> msg=<%s> (error=%s)",
			 OPRL,msg[0],ipcmsg,strerror(errno));
          }
        ipcmsg=rou_freestr(ipcmsg);
        }
      break;
    default	:	/*SAFE guard		*/
      proceed=false;
      break;
    }
  phase++;
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send an CKRCPT request to	*/
/*	mentor type server.			*/
/*						*/
/************************************************/
static void sendreqst(CTTYP *cta,char *msgref,char *command,char *rqst)

{
char *info;

info=rou_memstrcat((char *)0,"%s %s\n%s %s\n.",RQSTID,msgref,command,rqst);
(void) eml_printfcrlf(cta->tls,"%s",info);
info=rou_freestr(info);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to open a connexion to a	*/
/*	sender, to use to check if user is	*/
/*	existing or not				*/
/*						*/
/************************************************/
static CHKTYP *chkopenauxi(SKRTYP *skr,char *uniqid,CHKTYP *checker)

{
/*time to retry to connect to auxiliary		*/
#define	MXRTRY	300	

if (checker==(CHKTYP *)0) {
  static time_t lasttry=(time_t)0;
   
  time_t curtime;
  curtime=rou_systime();
  if ((lasttry+MXRTRY)<curtime) {
    checker=eml_openauxiliary(skr,uniqid);
    lasttry=curtime;
    }
  }
return checker;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to update a remote mentor	*/
/*	with a user status			*/
/*						*/
/************************************************/
static void refreshmentor(BOXTYP *boxe,char *caller,pid_t mnt,
			  RCPTMSG *rm,int chkstatus)

{
char *locmsg;

locmsg=rou_memstrcat((char *)0,"%s %s %s %s %s,%d %d %d",REFRESH,
				rm->from,rm->iprem,
				rm->rcpt,rm->listenip,rm->listenport,
				chkstatus,rm->xrelais);
/*ONEWAY*/
if (ipc_relaymsg(boxe,ONEWAY,mnt,locmsg)==0) 
  (void) alert(0,"%s Unable to relaymsg <%s> to checker (pid='%05d')",
		 "modchk.c:refreshmentor",locmsg,mnt);
locmsg=rou_freestr(locmsg);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send default value to	*/
/*	advice request while not handle by a	*/
/*	Mentor.					*/
/*						*/
/************************************************/
static int defltadv(BOXTYP *boxe,char *caller,ADVIMSG *advimsg,int slice)

{
if (advimsg!=(ADVIMSG *)0) {
  if (advimsg->doms!=(DOMCHK **)0) {
    char *advrly;
    int i;

    advrly=(char *)0;
    for (i=0;advimsg->doms[i]!=(DOMCHK *)0;i++) {
      if (advimsg->doms[i]->forwarded==false) {
	char *dmsg;

        (void) checkspam(advimsg,advimsg->doms[i]);
	if ((dmsg=eml_domtoadv(advimsg->doms[i]))!=(char *)0) {
	  advrly=rou_memstrcat(advrly,"%s",dmsg);
	  dmsg=rou_freestr(dmsg);
	  }
	}
      }
    if (advrly!=(char *)0) {
      slice++;
      advrly=rou_memstrcat(advrly,"%d %d/%d This was my advice",
				  CMDOK, slice,slice);
      if (ipc_postreply(boxe,caller,advrly)==0) {
	(void) alert(0,"%s Unable to post reply","modchk.c:defltadv");
	}
      advrly=rou_freestr(advrly);
      }
    }
  }
return slice;
}
/*

*/
/************************************************/
/*						*/
/*	Routine to find advice among sender or	*/
/*	checker related to domain.		*/
/*	limit set a the domain level.		*/
/*						*/
/************************************************/
static DISTYP **findadvice(BOXTYP *boxe,char *caller,ADVIMSG *advimsg,
			   SKRTYP **skrlst,DOMCHK *dom,int *slice,
			   DISTYP **msglst)

{
DOMTYP *skrs;
int phase;
int proceed;

skrs=(DOMTYP *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0		:	/*let get checker/sender	*/
      if ((skrs=eml_findskr(skrlst,dom->domain,false))==(DOMTYP *)0)
        proceed=false;
      break;
    case  1		:	/*do we have a checker?		*/
      if ((skrs->chkr1!=(SKRTYP *)0)&&(skrs->chkr1->skrpid!=(pid_t)0)) {
        char *dmsg;

        if ((dmsg=eml_domtomsg(dom))!=(char *)0) {
          DISTYP *msgck;

          if ((msgck=getdispatch(msglst,skrs->chkr1->skrpid))==(DISTYP *)0) {
            msgck=(DISTYP *)calloc(1,sizeof(DISTYP));
            msgck->chkpid=skrs->chkr1->skrpid;
            msgck->msg=strdup(advimsg->info);
            msglst=(DISTYP **)rou_addveclst((void **)msglst,(void *)msgck);
            (*slice)++;
            }
          msgck->msg=rou_memstrcat(msgck->msg,"%s",dmsg);
          dmsg=rou_freestr(dmsg);
          dom->forwarded=true;
	  }
	phase=999;		/*done, lets get out		*/
        }
      break;
    case  2		:	/*do we have a sender?		*/
      if ((skrs->sndr1!=(SKRTYP *)0)&&(skrs->sndr1->skrpid!=(pid_t)0)) {
	dom->dfs=skrs->sndr1->dfs;
	phase=999;		/*lets set the DFS info		*/
	}
      break;
    default	:		/*SAFE Guard			*/
      (void) free(skrs);
      proceed=false;
      break;
    }
  phase++;
  }
return msglst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a check by relay	*/
/*	request, purpose is to check if the 	*/
/*	originator is an acceptable recipient	*/
/*	from the relay SMTP daemon as the domain*/
/*	is unknown from clement while the flag	*/
/*	Relay is set.				*/
/*						*/
/************************************************/
static char *checkbyrelay(BOXTYP *boxe,CHKTYP *rely,char *caller,RCPTMSG *rm)

{
char *answ;
int chkstatus;
int phase;
int proceed;

answ=(char *)0;
chkstatus=ERRPROC;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*does relay accept?	*/
      if (rely!=(CHKTYP *)0) {
        chkstatus=usr_isexisting(rely,caller,rm);
	}
      break;
    default	:		/*SAFE guard	*/
      if (answ==(char *)0) {	/*always true	*/
	PLDTYP *pld;

        if ((pld=cac_getentry(rm->rcpt,pldsmstat))!=(PLDTYP *)0) {
	  answ=rou_memstrcat((char *)0,"%d %s",chkstatus,pld->info.smstat.str);
	  }
        else
          answ=strdup(eml_ascstat(chkstatus,rm->rqst));
	}
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a check by sender	*/
/*	request, purpose is to check if the 	*/
/*	orignator is an acceptable recipient	*/
/*	from the sender stand point.		*/
/*						*/
/************************************************/
static char *checkbysndr(BOXTYP *boxe,CHKTYP *sndr,char *caller,RCPTMSG *rm)

{
#define	OPKE	"mkdochk.c:checkbysndr,"

char *answ;
pid_t mnt;
pid_t relaysndr;
char *pids;
int chkstatus;
int phase;
int proceed;

answ=(char *)0;
mnt=(pid_t)0;
relaysndr=(pid_t)0;
pids=(char *)0;
chkstatus=ERRPROC;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0 	:	/*checkin sender available		*/
      if (sndr==(CHKTYP *)0) {
	answ=strdup(eml_ascstat(NOSNDR));
	proceed=false;
        }
      break;
    case  1 	:	/*extracting checker and relaysender	*/
      if ((pids=strrchr(rm->rqst,'|'))!=(char *)0) {
	*pids='\000';
	pids++;
	if (sscanf(pids,"%d %d",&mnt,&relaysndr)<2) {
	  *(pids-1)='|';
 	  (void) alert(0,"%s unable to sscanf request <%s> (bug?)",OPKE,rm->rqst);
	  answ=strdup(eml_ascstat(ERRPROC));
	  proceed=false;
	  }
        }
      break;

    case  2 	:	/*relay requested?			*/
      if (rm->xrelais==true)
	phase++;
      break;

    case  3 	:	/*no relay, lest check MX		*/
      if (eml_locatedfs(sndr->dfs,eml_strdomain(rm->rcpt))==(DFSTYP *)0) {
        if (dns_isgoodmx(eml_strdomain(rm->rcpt),rm->listenip)==false) {
          answ=strdup(eml_ascstat(NORELAY));
	  proceed=false;
	  }
	}
      break;

    case  4 	:	/*checking by sender			*/
      if (sndr!=(CHKTYP *)0)
        chkstatus=usr_isexisting(sndr,caller,rm);
      break;

    case  5	:	/*user is not good one, is it relayable?*/
      if (chkstatus!=CMDOK) {
	if ((rm->xrelais==true)&&(relaysndr!=(pid_t)0)) {
	  char *locmsg;
  
          locmsg=rou_memstrcat((char *)0,"%s %s",CKRRLY,rm->rqst);
          if (ipc_relaymsg(boxe,caller,relaysndr,locmsg)==0) 
            answ=strdup(eml_ascstat(ERRPROC,locmsg));
	  locmsg=rou_freestr(locmsg);
          proceed=false;/*lets relay process to take care	*/
	  }
	}
      break;

    case  6	:	/*do we have a sender ?			*/
      if (mnt==(pid_t)0)/*No					*/
	phase=999;	/*lets get out fast			*/
      break;
	
    case  7	:	/*We have a sender defined		*/
      switch (chkstatus) {
	case CMDOK  :	/*lets update the mentor user status	*/
	case BADPAR :
	case REJECT :
          (void) refreshmentor(boxe,caller,mnt,rm,chkstatus);
	  break;
	default	    :
	  break;
	}
      break;

    case  8	:	/*is this a Bypass			*/
      if (chkstatus==CMDOK) {
        if (strcasestr(rm->rcpt,POSTMASTER"@")==rm->rcpt)	
	  rm->bypass=true;
        if (strcasestr(rm->rcpt,ABUSE"@")==rm->rcpt)	
	  rm->bypass=true;
        if (rm->bypass==true) {
          answ=strdup(eml_ascstat(BYPASS,rm->rqst));
          proceed=false;/*we have an answer			*/
	  }
	}
      break;

    case  9	:	/*lets get advice from mentor		*/
      if (chkstatus==CMDOK) {
	char *locmsg;

        locmsg=rou_memstrcat((char *)0,"%s %s",CKRMNT,rm->rqst);
        if (ipc_relaymsg(boxe,caller,mnt,locmsg)==0) 
	  (void) alert(0,"%s Unable to relaymsg <%s> to mentor",OPKE,locmsg);
	else
	  proceed=false;/*Mentor will answer			*/
	locmsg=rou_freestr(locmsg);
	}
      break;

    default	:	/*SAFE guard				*/
      if (answ==(char *)0) {
	PLDTYP *pld;

        if ((pld=cac_getentry(rm->rcpt,pldsmstat))!=(PLDTYP *)0) {
	  answ=rou_memstrcat((char *)0,"%d %s",pld->info.smstat.status,
					       pld->info.smstat.str);
	  }
        else
          answ=strdup(eml_ascstat(chkstatus,rm->rqst));
	}
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a check by Mentor	*/
/*	request, purpose is to check if the 	*/
/*	orignator is an acceptable recipient	*/
/*	from the Mentor stand point.		*/
/*						*/
/************************************************/
static char *checkbymntr(BOXTYP *boxe,CHKTYP *mntr,char *caller,char *rqst)

{
char *answ;
int phase;
int proceed;

answ=(char *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*do we have a mentor?	*/
      if (mntr==(CHKTYP *)0) {	/*NO!		*/
	phase=999;	/*get out fast		*/
	}
      break;
    case  1	:	/*Yes we have a Mentor	*/
      if ((mntr->contact!=(CTTYP *)0)&&(mntr->contact->tls!=(TLSFILE *)0)) {
        (void) sendreqst(mntr->contact,caller,CKRCPT,rqst);
	proceed=false;
	}
      break;
    default	:	/*SAFE guard		*/
      answ=rou_memstrcat(answ,"%d",CMDOK);
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to split an authentication	*/
/*	message in 3 components.		*/
/*	username,password,uniqid.		*/
/*						*/
/************************************************/
static int splitmsg(char *msg,char **username,char **password,char **uniqid)

{
int good;

good=false;
*username=(char *)calloc(strlen(msg),sizeof(char));
*password=(char *)calloc(strlen(msg),sizeof(char));
*uniqid=(char *)calloc(strlen(msg),sizeof(char));
while (msg[0]==' ')
  (void) strcpy(msg,msg+1);
if (sscanf(msg,"%s %s %s",*username,*password,*uniqid)==3) {
  char *ptr;

  good=true;
  if ((ptr=strchr(*username,'<'))!=(char *)0) {
    good=false;
    (void) strcpy(*username,ptr+1);
    if ((ptr=strrchr(*username,'>'))!=(char *)0) {
      *ptr='\000';
      good=true;
      }
    }
  }
return good;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to send the temporary uniqid	*/
/*	to the checker to be stored within the	*/
/*	data base, if the check can't do it	*/
/*	message is not sent at all.		*/
/*						*/
/************************************************/
static void sendmyuniqid(CHLTYP *mng,char *username,char *uniqid,char **answ)

{
#define OPUN	"clement.c:sendmyuniqid,"
#define	MXTIME	30	/*waiting 30 seconds max*/

pid_t	chkrpid;


chkrpid=mng->pid[chl_checker];
if (chkrpid!=(pid_t)0) {
  char *locmsg;
  char *msgref;

  locmsg=rou_memstrcat((char *)0,"%s %s %s",UNIQID,username,uniqid);
  if ((msgref=ipc_postmsg(mng->boxe,chkrpid,locmsg))!=(char *)0) {
    char *got;
    time_t start;

    got=(char *)0;
    start=rou_systime();
    while (got==(char *)0) {
      (void) usleep(10000);
      if ((got=ipc_getreply(mng->boxe,msgref))==(char *)0) {
	if ((start+MXTIME)<rou_systime()) {
	  break;
	  }
	}
      }
    if (got==(char *)0) {
      (void) alert(0,"%s no answer transmiting uniqid for <%s>",
		      OPUN,username);
      *answ=rou_memstrcat(*answ,"%d Mentor Timeout",CMDOK);
      }
    else {
      *answ=got;
      }
    msgref=rou_freestr(msgref);
    }
  locmsg=rou_freestr(locmsg);
  }
else {
  /*Address provided is not within a know domain, no need to	*/
  /*to send the collected passwd.				*/
  *answ=rou_memstrcat(*answ,"%d",CMDOK);
  }
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to check user ID using PAM	*/
/*	or POP.					*/
/*						*/
/************************************************/
static char *docheckid(CHLTYP *mng,char *caller,char *rqst)

{
int chkstat;
char *answ;

chkstat=ERRPROC;
answ=(char *)0;
switch(rou_fork(false)) { 
  case  -1	:
    (void) alert(0,"%s,loc_emailmgs, Unable to start action process "
                   "(error=<%s>)","modchk.c",strerror(errno));
    answ=strdup(eml_ascstat(ERRPROC,""));
    break;
  case   0	: {
    char *username;
    char *password;
    char *uniqid; 

    username=(char *)0;
    password=(char *)0;
    uniqid=(char *)0;
    mng->pid[chl_checker]=getppid();
    if (splitmsg(rqst,&username,&password,&uniqid)==true) {
      switch (chkstat=uid_chkid(mng->chkidmod,username,password)) {
        case CMDOK	:
	  if (strcasecmp(uniqid,"void")!=0)
            (void) sendmyuniqid(mng,username,uniqid,&answ);
	  else	/*SMTP authentication	*/
	    {
            answ=strdup(eml_ascstat(IDOK));
	    }
	  break;
        default		:
          answ=strdup(eml_ascstat(BADID,"Authentication failure"));
	  break;
        }
      }
    else {
      answ=strdup(eml_ascstat(NOOP,answ));
      }
    if (ipc_postreply(mng->boxe,caller,answ)==0) {
      (void) alert(0,"%s Unable to post reply","modchk.c:docheckid");
      }
    answ=rou_freestr(answ);
    uniqid=rou_freestr(uniqid);
    password=rou_freestr(password);
    username=rou_freestr(username);
    (void) exit(0);
    }
    break;
  default	:	
    break;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle recipient checking	*/
/*	process					*/
/*						*/
/************************************************/
static char *checkingby(BOXTYP *boxe,CHKTYP *chkr,char *caller,
		        char *rqst,MENUM code)

{
char *answ;
RCPTMSG *rm;
int phase;
int proceed;

answ=(char *)0;
rm=(RCPTMSG *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*scanning message	*/
      if ((rm=eml_getrcptmsg(rqst))==(RCPTMSG *)0) {
	answ=strdup(eml_ascstat(BADPAR,rqst));
	proceed=false;
	}
      break;
    case 1	:	/*dispatching message	*/
      switch (code) {
        case msg_ckrsnd		:
          answ=checkbysndr(boxe,chkr,caller,rm);
          break;
        case msg_ckrmnt		:
          answ=checkbymntr(boxe,chkr,caller,rqst);
          break;
        case msg_ckrrly		:
          answ=checkbyrelay(boxe,chkr,caller,rm);
          break;
	default	:
	  (void) alert(0,"modechk.c:checkingby, unexpected code (bug?)");
	  answ=strdup(eml_ascstat(ERRPROC,rqst));
	  break;
        }
      break;
    default	:	/*SAFE guard		*/
      rm=eml_freercptmsg(rm);
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to rebounce MSMTP request	*/
/*	(CHKRCPT and ADVICE) to the remote	*/
/*	Mentor or if no such capability to a	*/
/*	local checker.				*/
/*						*/
/************************************************/
static int chkrebounce(BOXTYP *boxe,CHKTYP *checker)

{
#define	OPCR	"modchk.c:chkrebounce,"

int gotone;
char *caller;
char *rqst;
u_long msec;

gotone=false;
caller=(char *)0;
if ((rqst=ipc_getpost(boxe,&caller,&msec))!=(char *)0) {
  char *answ;
  MENUM code;

  answ=(char *)0;
  gotone=true;
  switch (code=getmsgcode(rqst)) {
    case msg_ckrsnd		:
    case msg_ckrmnt		:
    case msg_ckrrly		:
      answ=checkingby(boxe,checker,caller,rqst,code);
      break;
    case msg_ckrcpt		:
      (void) alert(0,"%s unexpected request <%s>",OPCR,rqst);
      answ=strdup(eml_ascstat(ERRPROC));
      break;
    case msg_refresh		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor)) {
        (void) sendreqst(checker->contact,caller,REFRESH,rqst);
	}
      break;
    case msg_goodmail		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor)) {
	(void) sendreqst(checker->contact,caller,GMAIL,rqst);
	}
      else {
	/*Facking a Mentor Acknowledge answer	*/
        answ=rou_memstrcat(answ,"%d MENTOR PROXY",CMDOK);
	}
      break;
    case msg_uniqid		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor)) {
	(void) sendreqst(checker->contact,caller,UNIQID,rqst);
	}
      else {
	/*Facking a Mentor Acknowledge answer	*/
        answ=rou_memstrcat(answ,"%d MENTOR PROXY",CMDOK);
	}
      break;
    case msg_advice		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor)) {
 	char *advice;
  
        advice=rou_memstrcat((char *)0,"%s %s",ADVICE,rqst);
        (void) eml_sendstream(checker->contact,RQSTID,caller,advice);
        advice=rou_freestr(advice);
	}
      else {
	ADVIMSG *advimsg;
	DFSTYP **dfs;

	dfs=(DFSTYP **)0;
        if (checker!=(CHKTYP *)0)
	  dfs=checker->dfs;
	if ((advimsg=eml_getadvimsg(rqst))!=(ADVIMSG *)0) {
          (void) defltadv(boxe,caller,advimsg,0);
  	  advimsg=eml_freeadvimsg(advimsg);
	  }
	}
      break;
    case msg_sent		:
      if ((checker!=(CHKTYP *)0)&&(checker->control==c_mentor))
	(void) sendreqst(checker->contact,caller,SENT,rqst);
      else
        (void) alert(0,"%s Unexpected sent request <%s>",
			OPCR,rqst);
      break;
    case msg_unknown		:
    default			:
      (void) alert(0,"%s request <%s> unknown",OPCR,rqst);
      answ=strdup(eml_ascstat(ERRPROC));
      break;
    }
  if (answ!=(char *)0) {
    if (ipc_postreply(boxe,caller,answ)==0) {
      (void) alert(0,"%s Unable to post reply",OPCR);
      }
    answ=rou_freestr(answ);
    }
  caller=rou_freestr(caller);
  rqst=rou_freestr(rqst);
  }
return gotone;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle checking request	*/
/*	submitted according domain list		*/
/*						*/
/************************************************/
static void dochecking(BOXTYP *boxe,SKRTYP *skr)

{
#define OPBI    "modchk.c:dochecking,"
#define KEEPACE	60	/*max idle time		*/

char *uniqid;

if ((uniqid=ipc_getonemsgref(boxe))!=(char *)0) {
  CHKTYP *checker;
  char *strpref;
  char **msg;
  int phase;

  checker=(CHKTYP *)0;
  strpref="Checking";
  if (skr->pref==p_checker) {
    strpref="Mentoring";
    }
  msg=(char **)0;
  phase=0;
  (void) journalling(true,APJRL,"%s on <%s> (process='%d')",
		         	strpref,skr->hostname,getpid());
  while (quickout==false) {
    char line[800];

    switch (phase) {
      case  0	:	/*starting connexion		*/
	checker=chkopenauxi(skr,uniqid,checker);
	break;
      case 1	:	/*checking if parent still up	*/
	if (rou_checkprocess(getppid())==false) {
          checker=eml_closeauxiliary(checker);
	  quickout=true;
	  }
	break;
      case 2	:	/*getting and forwarding request*/
	if (quickout==false) {
	  TIMESPEC timeout;

	  timeout.tv_sec=0;
	  timeout.tv_nsec=500000;
          if (ipc_waitpost(boxe,&timeout)==true) {
	    while (chkrebounce(boxe,checker)==true);
	    }
	  }
	break;
      case 3	:	/*updating user cache		*/
        (void) cac_expireentries();
	break;
      case 4	:	/*checker not up lets cycle	*/
	if (checker==(CHKTYP *)0) {
          (void) settitle("%s, Waiting for %s from %s",
			   appname,strpref,skr->hostname);
          phase=-1;
	  (void) sleep(1);
	  }
	break;
      case 5	:	/*collecting answer from Mentor*/
        (void) settitle("%s, %s done by %s",appname,strpref,skr->hostname);
	switch (checker->control) {
	  case c_local		:
	    break;
	  case c_standard	: 
	    /*standard SMTP remote, we do not care about input	*/
	    /*here, lets forget about it			*/
	    while (gettlsline(line,sizeof(line),checker->contact)!=(char *)0);
	    break;
	  case c_mentor		:
	    while (gettlsline(line,sizeof(line),checker->contact)!=(char *)0) {
              if ((msg==(char **)0)&&(strstr(line,ANSWID)!=line)) {
                break; /*got a disconnect from mentor */
		}
              msg=(char **)rou_addveclst((void **)msg,(void *)strdup(line));
              if ((strlen(line)==1)&&(line[0]=='.')) {
                (void) relaytoipc(boxe,msg);
                msg=rou_freestrlst(msg);
		break;	/*got all messages		*/
	        }
	      }
	    break;
	  }
	break;
      case 6	:	/*checking idle situation	*/
	phase=0;
	switch (checker->control) {
	  case c_local		:
	    break;
	  case c_standard	:
	  case c_mentor		:
	    if (eml_smtpidle(checker,KEEPACE)==true) {
              if (eml_sendcmd(checker->contact,CMDOK,RSET)!=CMDOK) {
	        phase=999;	/*trouble		*/
		break;
	        }
	      }
            if (tls_iseof(checker->contact->tls)==true)  {
	      phase=999;	/*trouble		*/
	      }
	    break;
	  }
	break;
      default	:	/*lets try again		*/
        checker=eml_closeauxiliary(checker);
        phase=-1;
	break;
      }
    phase++;
    }
  if (checker!=(CHKTYP *)0)
    checker=eml_closeauxiliary(checker);
  uniqid=rou_freestr(uniqid);
  (void) journalling(true,APJRL,"%s on <%s> now exiting (process='%d')",
		         	strpref,skr->hostname,getpid());
  }
else {
  (void) alert(0,"%s Unable to get uniqid",OPBI);
  }
(void) cac_cleanentries();
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to initiate checker/sender	*/
/*	process.				*/
/*						*/
/************************************************/
static pid_t startskr(BOXTYP *boxe,SKRTYP *skr)

{
#define	OPST	"modchk.c:startskr,"
pid_t child;

switch((child=rou_fork(false))) { 
  case  -1	:
    (void) alert(0,"%s, Unable to fork process (error=<%s>)",
		   OPST,strerror(errno));
    break;
  case   0	:	
    (void) dbg_setnoleak(2);
    memleak=DMTRACE;	/*memory leak tracking (xtrem debugging)*/
    (void) dochecking(boxe,skr);
    (void) dbg_dumpmem(appname,"startskr");
    (void) sleep(1);
    (void) exit(0);
    break;
  default	:
    break;
  }
return child;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to start or restart a 	*/
/*	checker/sender process.			*/
/*						*/
/************************************************/
static void checkskr(BOXTYP *boxe,SKRTYP **skrlst)

{
if (skrlst!=(SKRTYP **)0) {
  time_t curtime;
  int i;

  curtime=rou_systime();
  for (i=0;skrlst[i]!=(SKRTYP *)0;i++) {
    SKRTYP *sk;

    sk=skrlst[i];
    switch (sk->stat) {
      case s_new	:
      case s_down	:
	if ((sk->lastact+60)<curtime) {
	  sk->lastact=curtime;
	  sk->stat=s_active;
	  sk->skrpid=startskr(boxe,sk);
	  }
	break;
      case s_active	:
	if (kill(sk->skrpid,SIGCHECK)<0) {
	  sk->stat=s_down; 
	  }
	break;
      default		:
        (void) alert(0,"%s Unexpected skr=<%s> stat='%d'",
		        "modchk.c:checkskr,",sk->hostname,sk->stat);
	break;
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to stop all checker/sender	*/
/*	process.				*/
/*						*/
/************************************************/
static void dropskr(SKRTYP **skrlst)

{
int nbr;

if ((nbr=rou_vecsize((void **)skrlst))>0) {
  int i;
  int n;
  pid_t *pidlst;

  pidlst=(pid_t *)calloc(nbr,sizeof(pid_t));
  for (i=0,n=0;skrlst[i]!=(SKRTYP *)0;i++) {
    SKRTYP *sk;

    sk=skrlst[i];
    switch (sk->stat) {
      case s_active	:
	pidlst[n]=sk->skrpid;
	n++;
	break;
      default		:
	break;
      }
    }
  if (n>0) 
    (void) rou_childkill(pidlst,n,5);
  (void) free(pidlst);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a 'goodmail' info	*/
/*	Information is restransmitted to 	*/
/*	comain checker.				*/
/*						*/
/************************************************/
static char *dogoodmail(CHLTYP *mng,SKRTYP **skrlst,char *caller,char *rqst)

{
int status;
char *answ;
char *email;
DOMTYP *skrs;
int phase;
int proceed;

status=ERRPROC;
answ=(char *)0;
email=(char *)0;
skrs=(DOMTYP *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch(phase) {
    case  0	:	/*extracting Email	*/
      email=(char *)calloc(strlen(rqst),sizeof(char));
      if (sscanf(rqst,"%s",email)!=1) {
        (void) alert(0,"%s Unable to scan <%s>","modchk.c:dogoodmail,",rqst);
	phase=999;	/*get out fast		*/
	}
      break;
    case  1	:	/*finding checker	*/
      if ((skrs=eml_findskr(skrlst,eml_strdomain(email),false))==(DOMTYP *)0){
	status=NOMENT;
	phase=999;	/*get out fast		*/
	}
      break;
    case  2	:	/*sending to checker	*/
      status=CMDOK;
      if ((skrs->chkr1!=(SKRTYP *)0)&&(skrs->chkr1->skrpid!=(pid_t)0)) {
        char *locmsg;

        locmsg=rou_memstrcat((char *)0,"%s %s",GMAIL,rqst);
        if (ipc_relaymsg(mng->boxe,caller,skrs->chkr1->skrpid,locmsg)!=0) {
	  proceed=false;	/*feed back direct to caller	*/
	  }
	else {
	  /*we should do something if we are not able to	*/
	  /*transmit the goodmail info to checker (unlikely).	*/
	  /*Lets return CMDOK for now.				*/
	  }
	locmsg=rou_freestr(locmsg);
	}
      break;
    default	:	/*SAFE guard		*/
      answ=strdup(eml_ascstat(status,rqst));
      email=rou_freestr(email);      
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a checkrcpt		*/
/*	request, it try to find out what are the*/
/*	sender/checker related to to this	*/
/*	recipient E-mail.			*/
/*						*/
/************************************************/
static char *checkrcpt(CHLTYP *mng,IPTYP ***relais,SKRTYP **skrlst,
		       char *caller,char *rqst)

{
char *answ;
char *domain;
RCPTMSG *rm;
DOMTYP *skrs;
int phase;
int proceed;

answ=(char *)0;
domain=(char *)0;
rm=(RCPTMSG *)0;
skrs=(DOMTYP *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:		/*scanning request	*/
      if ((rm=eml_getrcptmsg(rqst))==(RCPTMSG *)0) {
	answ=strdup(eml_ascstat(BADPAR,rqst));
	proceed=false;
	}
      break;
    case   1    	:       /*refreshing relay list	*/
      *relais=net_mkiplst(mng->frelais,*relais,mng->lastload);
      rm->xrelais|=net_isiniplst(*relais,rm->iprem);
      break;
    case  2		:	/*is there any SKR ?	*/
      domain=eml_strdomain(rm->rcpt);
      if ((skrs=eml_findskr(skrlst,domain,rm->xrelais))==(DOMTYP *)0){
        answ=strdup(eml_ascstat(NORELAY,rm->rcpt));
	proceed=false;
	}
      break;
    case  3		:	/*forward to main sender*/
      if ((skrs->sndr1!=(SKRTYP *)0)&&(skrs->sndr1->skrpid!=(pid_t)0)) {
	char *locmsg;
	pid_t chkr1;
	pid_t sndr2;

	chkr1=(pid_t)0;
        sndr2=(pid_t)0;
	if (skrs->chkr1!=(SKRTYP *)0) {
	  chkr1=skrs->chkr1->skrpid;
	  }
	if (skrs->sndr2!=(SKRTYP *)0)
	  sndr2=skrs->sndr2->skrpid;
        locmsg=rou_memstrcat((char *)0,"%s %s %s %lu %s,%d %s %d|%05d %05d",
                                        CKRSND,rm->iprem,rm->from,
                                        rm->taille,
                                        rm->listenip,rm->listenport,
                                        rm->rcpt,rm->xrelais,
					chkr1,sndr2);
        if (ipc_relaymsg(mng->boxe,caller,skrs->sndr1->skrpid,locmsg)==0) {
          answ=strdup(eml_ascstat(ERRPROC,rqst));
	  }
	locmsg=rou_freestr(locmsg);
	phase=999;
	} 
      break;
    case  4		:	/*sending to secondary 	*/
      if ((skrs->sndr2!=(SKRTYP *)0)&&(skrs->sndr2->skrpid!=(pid_t)0)) {
	char *locmsg;

        locmsg=rou_memstrcat((char *)0,"%s %s %s %lu %s,%d %s %d",
                                       CKRRLY,rm->iprem,rm->from,
                                       rm->taille,
                                       rm->listenip,rm->listenport,
                                       rm->rcpt,rm->xrelais);
        if (ipc_relaymsg(mng->boxe,caller,skrs->sndr2->skrpid,locmsg)==0) {
          answ=strdup(eml_ascstat(ERRPROC,rqst));
	  }
	locmsg=rou_freestr(locmsg);
	phase=999;
	} 
      break;
    case  5		:	/*errproc status	*/
      answ=strdup(eml_ascstat(NOSNDR));
      (void) alert(0,"modchkrc.c:checkrcpt, No sender defined for <%s>",
		      domain);
      break;
    default		:	/*SAFE Guard		*/
      rm=eml_freercptmsg(rm);
      (void) free(skrs);
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send the SENT message to 	*/
/*	the proper checker, if there is no 	*/
/*	checker for the domain everything is	*/
/*	forgoten.				*/
/*						*/
/************************************************/
static void relaysent(BOXTYP *boxe,SKRTYP **skrlst,char *caller,char *rqst)

{
char **data;

if ((data=eml_splitmsg(rqst))!=(char **)0) {
  int i;
  SNDTYP **sndlst;

  sndlst=(SNDTYP **)0;
  for (i=0;data[i]!=(char *)0;i++) {
    char emailid[50];
    char chkname[100];
    char recipient[300];

    if (sscanf(data[i],"%95s %45s %295s",chkname,emailid,recipient)==3) {
      SNDTYP *chn;
      

      if ((chn=findsentlst(sndlst,chkname))==(SNDTYP *)0) {
	chn=(SNDTYP *)calloc(1,sizeof(SNDTYP));
	chn->chkname=strdup(chkname);
	sndlst=(SNDTYP **)rou_addveclst((void **)sndlst,(void *)chn);
        chn->lst=rou_memstrcat((char *)0,"%s %s",emailid,recipient);
	}
      else
        chn->lst=rou_memstrcat(chn->lst,"\n%s %s",emailid,recipient);
      }
    }
  if (sndlst!=(SNDTYP **)0) {
    int j;

    for (j=0;sndlst[j]!=(SNDTYP *)0;j++) {
      SKRTYP *skr;

      if ((skr=cnf_getskr(skrlst,sndlst[j]->chkname))!=(SKRTYP *)0) {
	if (skr->skrpid!=(pid_t)0) {
          char *locmsg;

          locmsg=rou_memstrcat((char *)0,"%s %s",SENT,sndlst[j]->lst);
	  /*ONEWAY*/
          (void) ipc_relaymsg(boxe,ONEWAY,skr->skrpid,locmsg);
	  locmsg=rou_freestr(locmsg);
	  }
	}
      sndlst[j]->lst=rou_freestr(sndlst[j]->lst);
      sndlst[j]->chkname=rou_freestr(sndlst[j]->chkname);
      (void) free(sndlst[j]);
      }
    (void) free(sndlst);
    }
  data=rou_freestrlst(data);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send the UNIQID to the	*/
/*	proper checker, if there is no checker	*/
/*	for the domain a plain CMDOK is returned*/
/*	(provided data is not relevant)		*/
/*						*/
/************************************************/
static char *relayuiqid(BOXTYP *boxe,SKRTYP **skrlst,char *caller,char *rqst)

{
char *answ;
DOMTYP *skrs;
int status;
int phase;
int proceed;
char email[300];

answ=(char *)0;
skrs=(DOMTYP *)0;
status=BADPAR;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*extracting reqtest domain	*/
      if (sscanf(rqst,"%s",email)!=1) {
        (void) alert(0,"modchk.c:relayuiqid, Unable to scan <%s>",rqst);
	proceed=false;
	}
      break;
    case  1	:	/*finding checker		*/
      if ((skrs=eml_findskr(skrlst,eml_strdomain(email),false))==(DOMTYP *)0){
	status=CMDOK;	/*no checker?, it is A_OK	*/
	phase=999;
	}
      break;
    case  2	:	/*do we have an active checker	*/
      status=CMDOK;
      if ((skrs->chkr1!=(SKRTYP *)0)&&(skrs->chkr1->skrpid!=(pid_t)0)) {
        char *locmsg;

        locmsg=rou_memstrcat((char *)0,"%s %s",UNIQID,rqst);
        if (ipc_relaymsg(boxe,caller,skrs->chkr1->skrpid,locmsg)!=0) {
	  proceed=false;
	  }
	else {
	  /*we should do something if we are not able to	*/
	  /*transmit the uniqid info to checker (unlikely).	*/
	  /*Lets return CMDOK for now.				*/
	  }
	locmsg=rou_freestr(locmsg);
	}
      break;
    default	:	/*SAFE guard			*/
      answ=strdup(eml_ascstat(status,rqst));
      proceed=false;
      break;
    }
  phase++;
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle a checkadvice	*/
/*	purpose is to find out what are the	*/
/*	checker in charge to give avdvice and	*/
/*	dispatch subrequest. Unassigned request	*/
/*	part got a default status.		*/
/*						*/
/************************************************/
static char *checkadv(BOXTYP *boxe,SKRTYP **skrlst,char *caller,char *rqst)

{
char *answ;
DISTYP **msglst;
ADVIMSG *advimsg;

answ=(char *)0;
msglst=(DISTYP **)0;
if ((advimsg=eml_getadvimsg(rqst))!=(ADVIMSG *)0) {
  int slice;	/*number of slice for the message	*/

  slice=0;
  if (advimsg->doms!=(DOMCHK **)0) {
    int i;

    for (i=0;advimsg->doms[i]!=(DOMCHK *)0;i++) {
      DOMCHK *domloc;

      domloc=advimsg->doms[i];
      msglst=findadvice(boxe,caller,advimsg,skrlst,domloc,&slice,msglst);
      }
    }
  /*now lets respond A_OK for none forwarded advice rqst	*/
  slice=defltadv(boxe,caller,advimsg,slice);
  if (msglst!=(DISTYP **)0) {
    int i;

    for (i=0;msglst[i]!=(DISTYP *)0;i++) {
      char *locmsg;

      locmsg=rou_memstrcat((char *)0,"%s %d/%d %s",
				     ADVICE,i+1,slice,msglst[i]->msg);
      if (ipc_relaymsg(boxe,caller,msglst[i]->chkpid,locmsg)==0) {
	/*we should do something if we are not able to transmit	*/
	/*the advice request to checker (unlikely).		*/
	(void) alert(0,"modchk.c:checkadv, unable to restransmit <%s>",locmsg);
	}
      locmsg=rou_freestr(locmsg);
      msglst[i]->msg=rou_freestr(msglst[i]->msg);
      (void) free(msglst[i]);
      }
    (void) free(msglst);
    }
  advimsg=eml_freeadvimsg(advimsg);
  }
return answ;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for SMTP request and	*/
/*	forward it to the proper sender or	*/
/*	checker.				*/
/*						*/
/************************************************/
static void chkcycle(CHLTYP *mng,IPTYP ***relais)

{
#define	OPCY	"modchk.c:chkcycle,"

SKRTYP **skrlst;

skrlst=cnf_getskrlst(mng->auxs);
while (quickout==false) {
  char *caller;
  char *rqst;
  u_long msec;
  TIMESPEC timeout;

  (void) rou_nozombie();
  msec=(u_long)0;
  timeout.tv_sec=1;
  timeout.tv_nsec=0;
  if (ipc_waitpost(mng->boxe,&timeout)==false) {
    (void) checkskr(mng->boxe,skrlst);
    continue;
    }
  if ((rqst=ipc_getpost(mng->boxe,&caller,&msec))!=(char *)0) {
    char *answ;
    MENUM code;

    answ=(char *)0;
    switch (code=getmsgcode(rqst)) {
      case msg_goodmail		:
        answ=dogoodmail(mng,skrlst,caller,rqst);
	break;
      case msg_ckrcpt		:
	answ=checkrcpt(mng,relais,skrlst,caller,rqst);
	break;
      case msg_advice		:
	answ=checkadv(mng->boxe,skrlst,caller,rqst);
	break;
      case msg_uniqid		:
	answ=relayuiqid(mng->boxe,skrlst,caller,rqst);
	break;
      case msg_sent		:
	(void) relaysent(mng->boxe,skrlst,caller,rqst);
	break;
      case msg_chkid		:
        answ=docheckid(mng,caller,rqst);
        break;
      case msg_ckrsnd		:
      case msg_ckrrly		:
      case msg_ckrmnt		:
	(void) alert(0,"%s Unexpected message='%d'",OPCY,code);
        answ=strdup(eml_ascstat(ERRPROC));
	break;
      case msg_unknown		:
      default			:
        (void) alert(0,"%s request <%s> unknown",OPCY,rqst);
        answ=strdup(eml_ascstat(ERRPROC));
        break;
      }
    if (answ!=(char *)0) {
      if (ipc_postreply(mng->boxe,caller,answ)==0) {
        (void) alert(0,"%s Unable to post reply",OPCY);
	}
      answ=rou_freestr(answ);
      }
    caller=rou_freestr(caller);
    rqst=rou_freestr(rqst);
    }
  }
(void) dropskr(skrlst);
skrlst=cnf_freeskrlst(skrlst);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to fork a process and start	*/
/*	wait for request to be check with a	*/
/*	local/remote sendmail.			*/
/*						*/
/************************************************/
pid_t chk_manager(CHLTYP *mng)

{
pid_t child;
IPTYP **relais;

relais=(IPTYP **)0;
switch((child=rou_fork(false))) {
  case  -1      :
    (void) alert(0,"modchk.c:chk_manager, Unable to start local process "
		   "(error=<%s>)",strerror(errno));
    break;
  case   0      :
    (void) journalling(true,APJRL,"%s (process='%d')",
				  "Checker control",getpid());
    (void) settitle("%s, Checker control",appname);
    memleak=DMTRACE;	/*memory leak tracking (xtrem debugging)*/
    (void) dbg_setnoleak(1);
    relais=net_dupiplst(mng->relais);
    (void) chkcycle(mng,&relais);
    relais=net_freeiplst(relais);
    (void) dbg_dumpmem(appname,"Checker");
    (void) sleep(1);
    (void) exit(0);
    break;
  default       :
    break;
  }
return child;
}
