/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	UNIDNS:					*/
/*	Take Care of all access to DNS data	*/
/*						*/
/************************************************/
#include	<string.h>
#include	<resolv.h> 
#include	<netdb.h> 
#include	<stdlib.h> 
#include	<ctype.h> 
#include	<errno.h> 
#include	<sys/stat.h>
#include	<arpa/inet.h>
#include	"unidns.h"
#include	"subrou.h"

#define	B64K	65536		/*according US Cert VU#738331	*/

typedef	union	{
        HEADER hdr;		/*defined in resolv.h 		*/
        u_char buf[B64K];	/*according US Cert VU#738331	*/
	}RSPTYP;
/*

*/
/************************************************/
/*						*/
/*	Procedure to query DNS information	*/
/*						*/
/************************************************/
static int myquery(const char *dname,int class,int type,
                   unsigned char *answer, int anslen)

{
#define	OPMQ	"unidns.c:myquery,"
int got;

if ((got=res_query(dname,class,type,answer,anslen))<0) {
  switch (errno) {
    case  0		:	/* do not care		*/
    case ETIMEDOUT	:	/* Connection timed out	*/
      break;
    default	:
      (void) alert(0,"%s query error for domain <%s> type='%d' "
		     "(error=<%d/%s>)",OPMQ,dname,type,errno,strerror(errno));
    }
  }
else {
  if (got>anslen) {
    (void) alert(0,"%s query too by for domain <%s> type='%d' (got=%d)",
		   OPMQ,dname,type,got);
    got=anslen;
    }
  }
return got;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extracte texte information	*/
/*	from DNS record.			*/
/*	SMTP client status.			*/
/*						*/
/************************************************/
static char **extracting(RSPTYP *rsp,int rsplen)

{
char **list;
int question;
int answer;
const unsigned char *msg;
const unsigned char *eom;
const unsigned char *cp;
int rc;
char buf[NS_PACKETSZ+1];

list=(char **)0;
question=ntohs(rsp->hdr.qdcount);
answer=ntohs(rsp->hdr.ancount);
msg=(unsigned char *)rsp;
eom=(unsigned char *)rsp+rsplen;
cp=(unsigned char *)rsp+sizeof(HEADER);
while ((question-->0)&&(cp<eom)) {
  if ((rc=dn_expand(msg,eom,cp,(char *)buf,sizeof(buf)-1))<0)
    break;
  cp +=rc+QFIXEDSZ;
  }
while((answer-->0)&&(cp<eom)) {

  if ((rc=dn_expand(msg,eom,cp,(char *)buf,sizeof(buf)-1))<0) {
    break;
    }
  if (rc>0) {
    int type;
    int class;
    int dlen;
    int ttl;
    int collected;
    const unsigned char *txtptr;
    char *dynbuf;

    cp+=rc;
    GETSHORT(type,cp);
    GETSHORT(class,cp);
    GETLONG(ttl,cp);
    GETSHORT(dlen,cp);
    collected=0;
    dynbuf=(char *)0;
    switch (type) {
      case T_MX		:
        if ((rc=dn_expand(msg,eom,cp+2,(char *)buf,sizeof(buf)-1))>0) {
          int pref;
	  char info[30];

	  pref=(((unsigned int)cp[0])<<8)|((unsigned int)cp[1]);
	  (void) snprintf(info,sizeof(info),"%d",pref);
          list=(char **)rou_addveclst((void **)list,(void *)strdup(info));
          list=(char **)rou_addveclst((void **)list,(void *)strdup(buf));
	  }
	break;
      case T_TXT	:
	txtptr=cp;
        dynbuf=(char *)calloc(10,sizeof(char));
	while (collected<dlen) {
          u_int taille;

	  (void) memset(buf,'\000',sizeof(buf));
          taille=((u_int)(txtptr[0]))&0X0FF; 
	  txtptr++;
          (void) snprintf(buf,sizeof(buf),"%.*s",taille,txtptr);
	  dynbuf=(char *)realloc(dynbuf,strlen(dynbuf)+strlen(buf)+5);
	  (void) strcat(dynbuf,buf);
	  collected+=taille+1;
	  txtptr+=taille;
          }
        list=(char **)rou_addveclst((void **)list,(void *)dynbuf);
	break;
      case T_A		:	{
        u_int32_t *addr;

        addr=(u_int32_t *)calloc(1,sizeof(u_int32_t));
        *addr=ntohl(*((u_int32_t *)cp));
        list=(char **)rou_addveclst((void **)list,(void *)addr);
	}
	break;
      case T_PTR	:
        if ((rc=dn_expand(msg,eom,cp,(char *)buf,sizeof(buf)-1))>0) {
          list=(char **)rou_addveclst((void **)list,(void *)strdup(buf));
	  }
	break;
      default		:
	break;
      }
    cp+=dlen;
    }
  }
return list;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to extract a hostname TXT	*/
/*	If the domain doesn't exist or domain	*/
/*	do not have a TXT, a void pointer is	*/
/*	returned.				*/
/*						*/
/*						*/
/************************************************/
char **dns_gettxt(char *hostname)

{
char **inftxt;
int answer;
RSPTYP rsp;

inftxt=(char **)0;
(void) memset(rsp.buf,'\000',sizeof(rsp.buf));
if ((answer=myquery(hostname,C_IN,T_TXT,rsp.buf,sizeof(rsp.buf)))>0){
  inftxt=extracting(&rsp,answer);
  }
return inftxt;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to extract a domain MX	*/
/*	If the domain doesn't exist or domain	*/
/*	do not have a MX, a void pointer is	*/
/*	returned.				*/
/*						*/
/************************************************/
MXTYP **dns_getmx(char *domain)

{
MXTYP **mxlist;

mxlist=(MXTYP **)0;
if ((domain!=(char *)0)&&(strlen(domain)>0)) {
  int answer;
  RSPTYP rsp;
  
  if ((answer=myquery(domain,C_IN,T_MX,rsp.buf,sizeof(rsp.buf)))>0){
    char **list;

    if (answer>=sizeof(rsp.buf))
      answer=sizeof(rsp.buf)-1;
    if ((list=extracting(&rsp,answer))!=(char **)0) {
      int i;

      for (i=0;list[i]!=(char *)0;i+=2) {
	MXTYP *mx;

	mx=(MXTYP *)calloc(1,sizeof(MXTYP));
	if ((list[i+1]!=(char *)0)&&(strlen(list[i+1])>0)) {
	  mx->preference=atoi(list[i]);
	  mx->mxname=strdup(list[i+1]);
	  mxlist=(MXTYP **)rou_addveclst((void **)mxlist,(void *)mx);
	  }
        }
      list=rou_freestrlst(list);
      }
    }
  }
return mxlist;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to free memory used by a	*/
/*	MXLIST.					*/
/*						*/
/************************************************/
MXTYP **dns_cleanmx(MXTYP **mxlist)

{
if (mxlist!=(MXTYP **)0) {
  int i;

  for (i=0;mxlist[i]!=(MXTYP *)0;i++) {
    if (mxlist[i]->mxname!=(char *)0)
      (void) free(mxlist[i]->mxname); 
    (void) free(mxlist[i]);
    }
  (void) free(mxlist);
  mxlist=(MXTYP **)0;
  }
return mxlist;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to return the PTR domain	*/
/*	from an ip number			*/
/*						*/
/************************************************/
char **dns_getptr(u_int32_t norig)

{
char **ptrlist;
int answer;
u_int32_t naddr;
RSPTYP rsp;
char raddr[300];

ptrlist=(char **)0;
naddr=htonl(norig);
(void) snprintf(raddr,sizeof(raddr),"%d.%d.%d.%d.in-addr.arpa",
				    ((naddr>>24)&0xFF),
				    ((naddr>>16)&0xFF),
				    ((naddr>>8)&0xFF),
				    ((naddr>>0)&0xFF));
if ((answer=myquery(raddr,C_IN,T_PTR,rsp.buf,sizeof(rsp.buf)))>0){
  ptrlist=extracting(&rsp,answer);
  }
return ptrlist;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to return the IP addr related	*/
/*	a hots name.				*/
/*						*/
/************************************************/
u_int32_t **dns_getaddr(char *domain)

{
u_int32_t **adrlist;

adrlist=(u_int32_t **)0;
if ((domain!=(char *)0)&&(strlen(domain)>0)) {
  int answer;
  RSPTYP rsp;
  
  if ((answer=myquery(domain,C_IN,T_A,rsp.buf,sizeof(rsp.buf)))>0){
    _res.options=RES_DEBUG;
    adrlist=(u_int32_t **)extracting(&rsp,answer);
    }
  }
return adrlist;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to clean memory used by	*/
/*	address list.				*/
/*						*/
/************************************************/
u_int32_t **dns_cleanaddr(u_int32_t **adrlist)

{
if (adrlist!=(u_int32_t **)0) {
  int i;

  for (i=0;adrlist[i]!=(u_int32_t *)0;i++) {
    (void) free(adrlist[i]);
    }
  (void) free(adrlist);
  adrlist=(u_int32_t **)0;
  }
return adrlist;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if an Ip (norig) is	*/
/*	part of an MX list			*/
/*						*/
/************************************************/
int dns_matchmx(char *domain,u_int32_t norig,int mask)

{
int match;
struct hostent *host;

match=false;
if ((host=gethostbyname(domain))!=(struct hostent *)0) {
  int i;

  for (i=0;host->h_addr_list[i]!=(char *)0;i++) {
    uint32_t naddr;
    u_int32_t nparam;

    (void) memcpy(&naddr,host->h_addr_list[i],host->h_length);
    nparam=ntohl(naddr);
    if (((nparam&(~0U<<mask))==(norig&(~0U<<mask)))==1) {
      match=true;
      break;
      }
    }
  }
return match;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to convert a string IPNUM to	*/
/*	normalize network number.		*/
/*						*/
/************************************************/
u_int32_t dns_convip(char *ipnum)

{
u_int32_t nip;
struct in_addr addrorig;

nip=(u_int32_t)0;
if (inet_aton(ipnum,&addrorig)==0)
  (void) alert(0,"%s unable to convert address <%s>",
		 "unidns.c:dns_convip,",ipnum);
else
  nip=ntohl(addrorig.s_addr);
return nip;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if the listen IP is 	*/
/*	a declared MX for a specific domain	*/
/*	Return false if not the case.		*/
/*						*/
/************************************************/
int dns_isgoodmx(char *domain,char *listenip)

{
int isgoodmx;
MXTYP **mxlst;

isgoodmx=false;
if ((mxlst=dns_getmx(domain))!=(MXTYP **)0) {
  int i;
  u_int32_t norig;

  norig=dns_convip(listenip);
  for (i=0;mxlst[i]!=(MXTYP *)0;i++) {
    if ((isgoodmx=dns_matchmx(mxlst[i]->mxname,norig,0))==true)
      break;
    }
  mxlst=dns_cleanmx(mxlst);
  }
return isgoodmx;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check a version number	*/
/*	against an expected one and the last	*/
/*	current one as reported via DNS.	*/
/*	Report false if version is not the same	*/
/*	and return the current one.		*/
/*						*/
/************************************************/
int dns_chkversion(char *domain,char *version,int taille)

{
int itsok;
char **txt;
char lastvers[100];

itsok=false;
(void) snprintf(lastvers,sizeof(lastvers),"%s","Unknown");
if ((txt=dns_gettxt(domain))!=(char **)0) {
  char *ptr;

  if ((ptr=strchr(txt[0],':'))!=(char *)0)
    *ptr='\000';
  (void) snprintf(lastvers,sizeof(lastvers),"%s",txt[0]);
  txt=rou_freestrlst(txt); 
  }
if (strcmp(version,lastvers)==0)
  itsok=true;
else
  (void) snprintf(version,taille,"%s",lastvers);
return itsok;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check a SAFE version number*/
/*	against an expected one and the last	*/
/*	current one as reported via DNS.	*/
/*	Report false if version is not the same	*/
/*	and return the current one.		*/
/*						*/
/************************************************/
int dns_chksafeversion(char *version,int taille)

{
int compare;
char *dnsversion;
char domain[200];

compare=0;
dnsversion=(char *)calloc(taille,sizeof(char));
(void) strncpy(dnsversion,version,taille);
(void) snprintf(domain,sizeof(domain),"version.%s.%s",appname,SAFEDOM);
if (dns_chkversion(domain,dnsversion,taille)!=true) {
  int i;
  int extension;
  float numvers[2];
  float diff;

  numvers[0]=0.0;
  numvers[1]=0.0;
  for (i=0;i<2;i++) {
    char *ptr;

    ptr=dnsversion;
    switch (i) {
      case  0	:
        break;
      case  1	:
        ptr=version;
        break;
      }
    if (sscanf(ptr,"%f-%d",&(numvers[i]),&extension)!=2)
      break;
    numvers[i]*=10000;
    numvers[i]+=extension;
    }
  diff=numvers[0]-numvers[1];
  if (diff!=0) {
    compare=-1;
    if (numvers[0]<numvers[1])
      compare=1;
    }
  (void) strncpy(version,dnsversion,taille);
  }
(void) free(dnsversion);
return compare;
}
