/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	UNIEML:					*/
/*	Take Care of all access low level SMTP	*/
/*	formating.				*/
/*						*/
/************************************************/
#include	<stdarg.h>
#include	<syslog.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	<malloc.h>
#include	<errno.h>
#include	<string.h>
#include	"subrou.h"
#include	"unieml.h"

#define	LOCKEXT	"-lock"

typedef	struct	{		/*stdinfo switcher	*/
	char *mstdout;		/*reference to stdout	*/
	char *mstderr;		/*reference to stdout	*/
	}STDTYP;
/*

*/
/************************************************/
/*						*/
/*	Procedure to find out the message number*/
/*	among a string of message		*/
/*						*/
/************************************************/
static char *getmsgnum(ADVIMSG *advi,char *msg)

{
int msgnum;
int msglast;
int p;

if (sscanf(msg,"%d/%d %n",&msgnum,&msglast,&p)==2) {
  advi->msgnum=msgnum;
  advi->msglast=msglast;
  msg+=p;
  }
return msg;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to locate the domain info	*/
/*	within a list				*/
/*						*/
/************************************************/
static DOMCHK *locatedom(DOMCHK **doms,char *domain)

{
DOMCHK *dom;

dom=(DOMCHK *)0;
if (doms!=(DOMCHK **)0) {
  int i;

  for (i=0;doms[i]!=(DOMCHK *)0;i++) {
    if (strcmp(doms[i]->domain,domain)==0) {
      dom=doms[i];
      break;
      }
    }
  }
return dom;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to plit a msg structure to	*/
/*	a line structure.			*/
/*						*/
/************************************************/
char **eml_splitmsg(char *msg)

{
char **lines;

lines=(char **)0;
while ((msg!=(char *)0)&&(msg[0]!='\000')) {
  char *ptr;

  if ((ptr=strchr(msg,'\n'))!=(char *)0) {
    *ptr='\000';
    if (strlen(msg)>0) {
      ptr--;
      if (*ptr=='\r')	/*to get rid of '\r\n' sequence	*/
	*ptr='\000';
      ptr++;
      }
    ptr++;
    }
  lines=(char **)rou_addveclst((void **)lines,strdup(msg));
  msg=ptr;
  }
return lines;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return the domain part	*/
/*	within an E-mail address		*/
/*	if no domain part found return the whole*/
/*	E-mail.					*/
/*						*/
/************************************************/
char *eml_strdomain(char *email)

{
char *ptr;

if ((ptr=strchr(email,'@'))!=(char *)0)
  ptr++;
else
  ptr=email;
return ptr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get to store a list of	*/
/*	string within the current journal	*/
/*						*/
/************************************************/
void eml_journal(char **datalogs)

{
if (datalogs!=(char **)0) {
  int i;

  for (i=0;datalogs[i]!=(char *)0;i++) {
    (void) journalling(false,APJRL,datalogs[i]);
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add a data to a list of	*/
/*	data to be logs later on.		*/
/*						*/
/************************************************/
char **eml_adddata(char **data,char *frm,...)

{
va_list args;
char strloc[1000];

va_start(args,frm);
(void) vsnprintf(strloc,sizeof(strloc),frm,args);
data=(char **)rou_addveclst((void **)data,(void *)strdup(strloc));
va_end(args);
return data;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get the ASCII meaning of	*/
/*	status according RFC 821 standard.	*/
/*						*/
/************************************************/
const char *eml_ascstat(int status,...)

{
static char result[1000];

va_list args;
char *strcmt;
char strloc[1000];

va_start(args,status);
strcmt="??";
switch (status) {
  case  CMDOK	:
    strcmt="Requested mail action okay, completed";
    break;
  case  RELAYOK	:
    strcmt="E-mail will be relayed";
    break;
  case  BYPASS	:
    strcmt="Please state the nature of the emergency!";
    break;
  case  QUITOK	:
    strcmt="%s closing connection";
    break;
  case  IDOK	:
    strcmt="OK Authenticated";
    break;
  case  NOVRFY	:
    strcmt="cannot VRFY user; try RCPT to attempt delivery (or try finger)";
    break;
  case TOOOFT 	:
    strcmt="(%s) SMTP client connecting too often sending spam, "
	   "please give me a break and retry later"; 
    break;
  case ALREAD 	:
    strcmt="You have a connection still in progress, "
	   "please give me a break and retry later";
    break;
  case ONHOLD  	:
    strcmt="Server not allowed to received mail for (%s)";
    break;
  case UNABLE  	:
    strcmt="Unable to proceed (%s)";
    break;
  case NEWSITE 	:
    strcmt="Checking your Clearance, please retry in few minutes";
    break;
  case NOSNDR 	:
    strcmt="Domain sender process not connected, please retry in few minutes";
    break;
  case NOCHKR 	:
    strcmt="Domain checker not available";
    break;
  case CHKUSR  	:
    strcmt="I need to double check recipient address, please retry in few minutes";
    break;
  case DATAOK  	:
    strcmt="Enter mail, end with \".\" on a line by itself";
    break;
  case TOOMANY  	:
    strcmt="Too many mail pending for (%s) retry in few minutes";
    break;
  case  SYNERR	:
    strcmt="syntax error, command unrecognized...(%s)";
  case  BADPAR	:
    strcmt="syntax error in parameters or arguments (%s)";
    break;
  case  CMDBAD	:
    strcmt="command not implemented (%s)";
    break;
  case  BADSEQ	:
    strcmt="%s (Bad sequence of commands)";
    break;
  case  BADAUTH	:
    strcmt="AUTH mechanism %s not available";
    break;
  case  BADHDR	:
    strcmt="E-mail parsing header error";
    break;
  case  NOOP	:
    strcmt="Sorry, we do not allow this operation"; 
    break;
  case  STOLEN	:
    strcmt="Sorry, request denied (%s)"; 
    break;
  case  DOMNRES	:
    strcmt="(%s) domain must resolve";
    break;
  case  REJECT	:
    strcmt="Requested mail action not taken: mailbox unavailable";
    break;
  case  NOLOC	:
    strcmt="Requested mail action not taken: user not local";
    break;
  case  MALABRT	:
    strcmt="Requested mail action aborted (%s)";
    break;
  case  BADID	:
    strcmt="authentication failed";
    break;
  case  EXPIRED	:
    strcmt="Your contract is now expired";
    break;
  case  CMDABRT	:
    strcmt="Requested mail action aborted: (%s)";
    break;
  case  FAILED	:
    strcmt="transaction failed (%s)";
    break;
  case  NORELAY	:
    strcmt="Requested mail action aborted: we do not relay";
    break;
  case  NOMENT	:
    strcmt="Domain (%s) not serviced";
    break;
  case  ISBIG	:
    strcmt="Email is found too big by at least one recipient";
    break;
  case  NDIRTY 	:
  case  NOSPAMS	:
    strcmt="ATTENTION!!! %s, PLEASE CLICK ON THIS LINK TO BE VALIDATED. "
	   "(Your E-mail has been detected as coming from a Spamming server. "
	   "In order to accept your first emails we need a validation)";
    (void) snprintf(strloc,sizeof(strloc),"%d %s",status,strcmt);
    (void) vsnprintf(result,sizeof(result),strloc,args);
    strcmt=(char *)0;
    break;
  case  BADMX	:
    strcmt="Requested mail action not taken (%s)";
    break;
  case  BADSPF	:
    strcmt="Requested mail action not taken "
	   "(SPF: Server not allowed to send email for this domain)";
    break;
  case  NORESP	:
    strcmt="Requested mail action not taken: No daemon";
    break;
  case  OPRELAY	:
    strcmt="Requested mail action aborted (%s/[%s] detected as an open relay)";
    break;
  case  BADVIR	:
    strcmt="Requested mail action aborted (virus (%s) in the E-mail)";
    break;
  case  MANYRJT	:
    strcmt="Too many reject already, aborting connection";
    break;
  default	:
    (void) alert(0,"unieml.c,eml_ascstat: status '%d' unexpected",status);
  case  ERRPROC	:
    strcmt="Requested action aborted, local error in processing";
    break; 
  }
if (strcmt!=(char *)0) {
  (void) snprintf(strloc,sizeof(strloc),"%d %s",status,strcmt);
  (void) vsnprintf(result,sizeof(result),strloc,args);
  }
va_end(args);
return result;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract the listening port	*/
/*	number if present in the listening IP	*/
/*	return a message structure		*/
/*						*/
/************************************************/
int eml_getlistenport(char *listenip)

{
int listenport;
char *ptr;

listenport=SMTPPORT;
if ((ptr=strchr(listenip,','))!=(char *)0) {
  *ptr='\000';
  listenport=atoi(ptr+1);
  }
return listenport;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to split an CKRCPT message and*/
/*	return a message structure		*/
/*						*/
/************************************************/
RCPTMSG *eml_getrcptmsg(char *msg)

{
RCPTMSG *rcptmsg;
char *from;
char *rcpt;
unsigned long taille;
int relais;
char iprem[50];
char listenip[50];

rcptmsg=(RCPTMSG *)0;
from=(char *)calloc(strlen(msg),sizeof(char));
rcpt=(char *)calloc(strlen(msg),sizeof(char));
if (sscanf(msg,"%45s %s %lu %45s %s %d",
	        iprem,from,&taille,listenip,rcpt,&relais)==6) {
  rcptmsg=(RCPTMSG *)calloc(1,sizeof(RCPTMSG));
  rcptmsg->rqst=strdup(msg);
  rcptmsg->iprem=strdup(iprem);
  rcptmsg->listenport=eml_getlistenport(listenip);
  rcptmsg->listenip=strdup(listenip);
  rcptmsg->taille=taille;
  rcptmsg->from=strdup(from);
  rcptmsg->rcpt=strdup(rcpt);
  if (relais>0)
    rcptmsg->xrelais=true;
  }
else {
  (void) alert(0,"unieml.c:eml_getrcptmsg, Unable to scan <%s>",msg);
  }
(void) free(rcpt);
(void) free(from);
return rcptmsg;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by a 	*/
/*	CKRCPT message type structure		*/
/*						*/
/************************************************/
RCPTMSG *eml_freercptmsg(RCPTMSG *rcptmsg)

{
if (rcptmsg!=(RCPTMSG *)0) {
  rcptmsg->rcpt=rou_freestr(rcptmsg->rcpt);
  rcptmsg->from=rou_freestr(rcptmsg->from);
  rcptmsg->listenip=rou_freestr(rcptmsg->listenip);
  rcptmsg->iprem=rou_freestr(rcptmsg->iprem);
  rcptmsg->rqst=rou_freestr(rcptmsg->rqst);
  (void) free(rcptmsg);
  rcptmsg=(RCPTMSG *)0;
  }
return rcptmsg;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to split an ADVICE message and*/
/*	return a message structure		*/
/*						*/
/************************************************/
ADVIMSG *eml_getadvimsg(char *msg)

{
typedef	enum	{
	g_rcpt,		/*RCPT entry		*/
	g_ofrom,	/*Official Header From	*/
	g_spam,		/*SPAM level		*/
	g_info,		/*Email info		*/
	g_noise		/*discarded entry	*/
	}GTYP;

static VOCTYP vocinfo[]={
		{g_rcpt,ADVRCPT,(const void *)0},
		{g_ofrom,HFROM,(const void *)0},
		{g_spam,SPAM,(const void *)0},
		{g_info,MSGINF,(const void *)0},
		{g_noise,(char *)0,(const void *)0},
		};
ADVIMSG *adv;
char **data;

adv=(ADVIMSG *)calloc(1,sizeof(ADVIMSG));
msg=getmsgnum(adv,msg);
adv->spmlim=9999.9;
if ((data=eml_splitmsg(msg))!=(char **)0) {
  int i;

  for (i=0;data[i]!=(char *)0;i++) {
    char *addtoinfo;
    int dmy;

    addtoinfo=strdup(data[i]);
    dmy=0;
    switch (rou_getvoca(vocinfo,data[i])->code) {
      case g_ofrom	:
        adv->hdom=strdup(data[i]);
	break;
      case g_spam	:
	(void) sscanf(data[i],"%f/%f",&(adv->spmlvl),&(adv->spmlim));
	break;
      case g_info	:
	(void) sscanf(data[i],"%d %d %d %d",&dmy,&dmy,&dmy,&(adv->relayable));
	break;
      case g_rcpt	:
	addtoinfo=rou_freestr(addtoinfo);
	if (strlen(data[i])>0) {
          int pos;
	  int status;
	  int sf;
          char email[1000];

          if (sscanf(data[i],"%d <%999[^>]> %d%n",&pos,email,&status,&sf)==3){
 	    RCPADV *radv;
	    char *domain;
	    DOMCHK *dom;
  
	    domain="external";
	    if ((domain=strchr(email,'@'))!=(char *)0)
	      domain++;
	    if ((dom=locatedom(adv->doms,domain))==(DOMCHK *)0) {
	      dom=(DOMCHK *)calloc(1,sizeof(DOMCHK));
	      dom->domain=strdup(domain);
	      adv->doms=(DOMCHK **)rou_addveclst((void **)adv->doms,
						 (void *)dom);
              }
	    radv=(RCPADV *)calloc(1,sizeof(RCPADV));
	    radv->pos=pos;
	    radv->email=strdup(email);
	    radv->status=status;
	    radv->why=strdup(data[i]+sf);
            dom->rcpts=(RCPADV **)rou_addveclst((void **)dom->rcpts,
						(void *)radv);
	    }
	  }
	break;
      case g_noise	:
	break;
      }
    if (addtoinfo!=(char *)0) {
      adv->info=rou_memstrcat(adv->info,"%s\n",addtoinfo);
      addtoinfo=rou_freestr(addtoinfo);
      }
    }
  data=rou_freestrlst(data);
  }
return adv;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by	*/
/*	an ADVIMSG structure			*/
/*						*/
/************************************************/
ADVIMSG *eml_freeadvimsg(ADVIMSG *advimsg)

{
if (advimsg!=(ADVIMSG *)0) {
  if (advimsg->doms!=(DOMCHK **)0) {
    int i;

    for (i=0;advimsg->doms[i]!=(DOMCHK *)0;i++) {
      DOMCHK *dom;

      dom=advimsg->doms[i];
      if (dom->rcpts!=(RCPADV **)0) {
	int j;

        for (j=0;dom->rcpts[j]!=(RCPADV *)0;j++) {
          dom->rcpts[j]->why=rou_freestr(dom->rcpts[j]->why);
          dom->rcpts[j]->email=rou_freestr(dom->rcpts[j]->email);
          (void) free(dom->rcpts[j]);
	  }
        (void) free(dom->rcpts);
	}
      dom->domain=rou_freestr(dom->domain);
      (void) free(dom);
      }
    (void) free(advimsg->doms);
    }
  advimsg->hdom=rou_freestr(advimsg->hdom);
  advimsg->info=rou_freestr(advimsg->info);
  (void) free(advimsg);
  advimsg=(ADVIMSG *)0;
  }
return advimsg;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to format a DOMCHK as a msg	*/
/*	component.				*/
/*						*/
/************************************************/
char *eml_domtomsg(DOMCHK *dom)

{
char *msg;

msg=(char *)0;
if (dom!=(DOMCHK *)0) {
  if (dom->rcpts!=(RCPADV **)0) {
    int i;

    for (i=0;dom->rcpts[i]!=(RCPADV *)0;i++) {
      RCPADV *adv;
      int smtpstatus;
                                                                                
      adv=dom->rcpts[i];
      smtpstatus=adv->status;
      msg=rou_memstrcat(msg,"%s %02d <%s> %d %s\n",ADVRCPT,
                             adv->pos,adv->email,smtpstatus,adv->why);
      }
    }
  }
return msg;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to format a DOMCHK as an	*/
/*	ADVICE respond message.			*/
/*	advices is the number of advice given.	*/
/*						*/
/************************************************/
char *eml_domtoadv(DOMCHK *dom)

{
char *msg;

msg=(char *)0;
if (dom!=(DOMCHK *)0) {
  dom->forwarded=true;
  if (dom->rcpts!=(RCPADV **)0) {
    int i;
                                                                                
    for (i=0;dom->rcpts[i]!=(RCPADV *)0;i++) {
      RCPADV *adv;
      int locstat;
      char locadv[30];

      adv=dom->rcpts[i];
      locstat=adv->status;
      (void) strcpy(locadv,"");
      switch(locstat) {
	case RLYADV	:	/*RELAYING E-mail	*/
	  locstat=SENADV;	/*send E-mail		*/
	  break;
	case GIVADV	:
	  (void) snprintf(locadv,sizeof(locadv),"%s,Spl=%4.2f",
						"LOC",adv->spamlim);
	  if (adv->spam==false)
	    locstat=SENADV;	/*send E-mail		*/
	  else 
	    locstat=QUAADV;	/*E-mail in quarantine	*/
	  break;
	case DISADV	:	/*Discarding anyway	*/
	  break;
	default		:
	  (void) alert(0,"unieml.c:eml_domtoadv, Unexpected advice "
			 "value='%d' for Email=<%s>",
			  locstat,adv->email);
	  break;
	}
      msg=rou_memstrcat(msg,"%03d-%02d %s\n",locstat,adv->pos,locadv);
      }
    }
  }
return msg;
}
/*

*/
/************************************************/
/*						*/
/*	procedure to switch both stdout and	*/
/*	stderr to a log file file.		*/
/*						*/
/************************************************/
STDDBG *eml_switchstd(char *name)

{
#define	MXSTD	200

time_t curtime;
STDTYP *swlog;

curtime=rou_systime();
swlog=(STDTYP *)calloc(1,sizeof(STDTYP));
swlog->mstdout=(char *)calloc(MXSTD,sizeof(char));
(void) rou_mkname(swlog->mstdout,MXSTD,d_spool,DBGSDIR,name,".stdout");
if ((stdout=freopen(swlog->mstdout,"a",stdout))==(FILE *)0) {
  (void) code_fault("%s:switchstd, Unable to fopen stdout file '%s' "
		    "(error <%s>)",appname,swlog->mstdout,strerror(errno));
  }
(void) fprintf(stdout,"-Log open at %s\n",rou_getstrdatetime(curtime));
(void) fflush(stdout);
if (verbose==false) {
  /*needed for Clamav report!                             	*/
  swlog->mstderr=(char *)calloc(MXSTD,sizeof(char));
  (void) rou_mkname(swlog->mstderr,MXSTD,d_spool,DBGSDIR,name,".stderr");
  if ((stderr=freopen(swlog->mstderr,"a",stderr))==(FILE *)0) {
    (void) code_fault("%s:switchstd, Unable to reopen stderr file '%s'"
		      "(error <%s>)",appname,swlog->mstderr,strerror(errno));
    }
  (void) fprintf(stderr,"-Log open at %s\n",rou_getstrdatetime(curtime));
  (void) fflush(stderr);
  }
return (STDDBG *)swlog;
}
/*

*/
/************************************************/
/*						*/
/*	procedure to close both stdout and	*/
/*	stderr making sure trace file are 	*/
/*	properly closed.			*/
/*						*/
/************************************************/
STDDBG *eml_closeswitchstd(STDDBG *stdinfo)

{
STDTYP *swlog;
time_t curtime;

swlog=(STDTYP *)stdinfo;
curtime=rou_systime();
if (swlog!=(STDTYP *)0) {
  (void) fprintf(stdout,"-Log close at %s\n",rou_getstrdatetime(curtime));
  (void) fclose(stdout);
  (void) free(swlog->mstdout);
  if (verbose==false) {
    (void) fprintf(stderr,"-Log close at %s\n",rou_getstrdatetime(curtime));
    (void) fclose(stderr);
    (void) free(swlog->mstderr);
    }
  (void) free(swlog);
  stdinfo=(STDDBG *)0;
  }
return stdinfo;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to locate a domain name in a	*/
/*	domain list.				*/
/*						*/
/************************************************/
int eml_locatedomain(char **domains,char *dom)

{
int found;

found=false;
if ((domains!=(char **)0)&&(dom!=(char *)0)) {
  int i;

  for (i=0;domains[i]!=(char *)0;i++) {
    if (strcmp(domains[i],dom)==0) {
      found=true;
      break;
      }
    }
  }
return found;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to free memory used by RCPTYP	*/
/*	deinition.				*/
/*						*/
/************************************************/
RCPTYP *eml_freercpt(RCPTYP *rcpt)

{
if (rcpt!=(RCPTYP *)0) {
  rcpt->why=rou_freestr(rcpt->why);
  rcpt->rcpt=rou_freestr(rcpt->rcpt);
  (void) free(rcpt);
  rcpt=(RCPTYP *)0;
  }
return rcpt;
}
/*
^L
*/
/************************************************/
/*						*/
/*      Procedure to clean end of line from	*/
/*	any \r\n character			*/
/*						*/
/************************************************/
char *eml_cleanendline(char *line)

{
int i;

i=strlen(line)-1; 
while (i>=0) {
  if ((line[i]!='\r')&&(line[i]!='\n'))
    break;
  line[i]='\000';
  i--;
  }
return line;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to create a DFSTYP structure.	*/
/*						*/
/************************************************/
DFSTYP *eml_mkdf(char *domain,char *webgood,float spamlim,
		 int qualoc,char *quahead,int quaextra)

{
DFSTYP *dfs;

dfs=(DFSTYP *)calloc(1,sizeof(DFSTYP));
dfs->domain=strdup(domain);
if (webgood!=(char *)0)
  dfs->webgood=strdup(webgood);
dfs->spamlim=spamlim;
dfs->qualoc=qualoc;
dfs->quahead=strdup(quahead);
dfs->quaextra=quaextra;
return dfs;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free one DFSTYP structure.	*/
/*						*/
/************************************************/
DFSTYP *eml_freeonedfs(DFSTYP *dfs)

{
if (dfs!=(DFSTYP *)0) {
  dfs->quahead=rou_freestr(dfs->quahead);
  dfs->webgood=rou_freestr(dfs->webgood);
  dfs->domain=rou_freestr(dfs->domain);
  (void) free(dfs);
  dfs=(DFSTYP *)0;
  }
return dfs;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free a DFSTYP structure	*/
/*	list.					*/
/*						*/
/************************************************/
DFSTYP **eml_freedfslst(DFSTYP **dfs)

{
if (dfs!=(DFSTYP **)0) {
  int i;

  for (i=0;dfs[i]!=(DFSTYP *)0;i++) 
    dfs[i]=eml_freeonedfs(dfs[i]);
  (void) free(dfs);
  dfs=(DFSTYP **)0;
  }
return dfs;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to check if a domain pertain	*/
/*	to a DFS list, return pointer to DFS	*/
/*						*/
/************************************************/
DFSTYP *eml_locatedfs(DFSTYP **dfs,char *domain)

{
DFSTYP *df;

df=(DFSTYP *)0;
if (dfs!=(DFSTYP **)0) {
  int i;

  for (i=0;dfs[i]!=(DFSTYP *)0;i++) {
    if (strcmp(dfs[i]->domain,domain)==0) {
      df=dfs[i];
      break;
      }
    }
  }
return df;
}
#ifdef	NEWCODE
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to lock a sender to avoid to	*/
/*	receive E-mail while the sending queue	*/
/*	is already above acceptable limit	*/
/*						*/
/************************************************/
int eml_locksender(char *sender,int action)

{
int done;
char filename[300];

done=true;
(void) rou_mkname(filename,sizeof(filename),d_lock,LOCKDIR,sender,LOCKEXT);
if (action==false) 
  (void) unlink(filename);
else {
  int handle;

  if ((handle=creat(filename,0644))<0)
    (void) core_dump("Unable to creat lock file <%s> (error=<%s>)",
                      filename,strerror(errno));
  (void) close(handle);
  }
return done;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to check if a sender channel	*/
/*	is locked, such we can refuse E-mail and*/
/*	not overload the sending queue.		*/
/*						*/
/************************************************/
int eml_chksender(char *sender)

{
int lock;
struct stat bufstat;
char filename[300];

lock=true;
(void) rou_mkname(filename,sizeof(filename),d_lock,LOCKDIR,sender,LOCKEXT);
if (stat(filename,&bufstat)<0) {
  lock=false;
  }
return lock;
}
#endif
