/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	GESCNF:					*/
/*	Take Care of all config definition	*/
/*						*/
/************************************************/
#include	<malloc.h>
#include	<syslog.h>
#include	<stdio.h>
#include	<string.h>
#include	<unistd.h>
#include	<ctype.h>
#include	<errno.h>
#include	<netdb.h>
#include	<sys/stat.h>
#include	<arpa/inet.h>
#include	"subrou.h"
#include	"uninet.h"
/*

*/
/************************************************/
/*						*/
/*	Procedure to create a relay struct	*/
/*						*/
/************************************************/
static IPTYP *mkiplst(struct in_addr addrparam,int mask)

{
IPTYP *ipl;

ipl=(IPTYP *)calloc(1,sizeof(IPTYP));
ipl->ip=ntohl(addrparam.s_addr);
ipl->mask=32-mask;
return ipl;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to scan ip info and return	*/
/*	ONE ip information struct		*/
/*						*/
/************************************************/
static IPTYP **scanip(char *info)

{
#define	OPSC	"uninet.c:scanip,"

IPTYP **iplst;
char *ptr;
int mask;
char ip[30];

iplst=(IPTYP **)0;
mask=32;
(void) strcpy(ip,"");
if ((ptr=strchr(info,'['))!=(char *)0) {
  ptr++;
  info=ptr;
  if ((ptr=strchr(info,']'))!=(char *)0) 
    *ptr='\000';
  }
if (isdigit(info[0])!=0) {
  if (sscanf(info,"%29[^/]/%d",ip,&mask)>=1) {
    struct in_addr addrparam;

    if ((inet_aton(ip,&addrparam)!=0)&&((mask>0)&&(mask<=32))) {
      iplst=(IPTYP **)rou_addveclst((void **)iplst,
				    (void *)mkiplst(addrparam,mask));
      }
    else {
      (void) alert(0,"%s <%s> is not an IP address",OPSC,info);
      }
    }
  else {
    (void) alert(0,"%s can't scan <%s>",OPSC,info);
    }
  }
else {	/*it is a name rather an IP		*/
  struct hostent *h;

  if ((h=(struct hostent *)gethostbyname(info))==(struct hostent *)0) {
    (void) alert(2,"%s Unable to get '%s' host IP (error='%s')",
		 OPSC,info,strerror(errno));
    }
  else {
    int i;

    for (i=0;h->h_addr_list[i]!=(char *)0;i++) {
      struct in_addr addrparam;

      (void) memcpy(&addrparam,h->h_addr_list[i],h->h_length);
      iplst=(IPTYP **)rou_addveclst((void **)iplst,
				    (void *)mkiplst(addrparam,32));
      }
    }
  }
return iplst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add relay information as a	*/
/*	relay list.				*/
/*						*/
/************************************************/
IPTYP **net_addiplst(IPTYP **iplst,char *info)

{
char *scan;

scan=info;
while (strlen(scan)>0) {
  IPTYP **liplst;
  char *ptr;
 
  liplst=(IPTYP **)0; 
  if ((ptr=strchr(scan,':'))!=(char *)0)
    *ptr='\000';
  if ((liplst=scanip(scan))!=(IPTYP **)0) {
    int i;

    for (i=0;liplst[i]!=(IPTYP *)0;i++) 
      iplst=(IPTYP **)rou_addveclst((void **)iplst,(void *)liplst[i]); 
    (void) free(liplst);
    }
  scan="";
  if (ptr!=(char *)0)
    scan=ptr+1;
  }
return iplst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if we need to update	*/
/*	the relais list with a new one updated	*/
/*						*/
/************************************************/
IPTYP **net_mkiplst(char **filelst,IPTYP **iplst,time_t lastload)

{
#define	OPCK	"uninet.c:net_mkiplst"

static time_t lastone=(time_t)0;

if (filelst!=(char **)0) {
  time_t curtime;
  int torefresh;
  int i;
  
  curtime=rou_systime();
  torefresh=(lastload>lastone);
  for (i=0;filelst[i]!=(char *)0;i++) {
    switch (rou_isnewfile(filelst[i],lastone)) {
      case  -1	:	/*file missing			*/
        break;
      case   0	:	/*no update			*/
        break;
      case   1	:	/*update done, lets refresh	*/
        torefresh|=true;
        break;
       }
     }
  if (torefresh==true) {
    iplst=net_freeiplst(iplst);
    for (i=0;filelst[i]!=(char *)0;i++) {
      FILE *fichier;
      int phase;
      int proceed;
      char line[500];
  
      fichier=(FILE *)0;
      phase=1;
      proceed=true;
      while (proceed==true) {
        switch (phase) {
          case  0	:	/*phase error				*/
	    proceed=false;
	    break;
          case  1	:	/*Lets open the file			*/
            if ((fichier=fopen(filelst[i],"r"))==(FILE *)0) {
              (void) alert(0,"%s Unable to open file <%s> (error=<%s>)",
		             OPCK,filelst[i],strerror(errno));
	      phase=-1;
	      }
	    break;
          case  2	:	/*Lets load the relay list		*/
            while (rou_getstr(fichier,line,sizeof(line),'#')!=(char *)0) {
	      char *ptr;
    
	      if ((ptr=strchr(line,'\n'))!=(char *)0)
	        *ptr='\000';
	      iplst=net_addiplst(iplst,line);
              }
            (void) fclose(fichier);
	    break;
          default	:	/*SAFE case				*/
	    proceed=false;
	    break;
          }
        phase++;
        }
      }
    lastone=curtime;
    }
  }
return iplst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to duplicate iplist contents	*/
/*	list.					*/
/*						*/
/************************************************/
IPTYP **net_dupiplst(IPTYP **iplst)

{
IPTYP **dupip;

dupip=(IPTYP **)0;
if (iplst!=(IPTYP **)0) {
  int i;

  for (i=0;iplst[i]!=(IPTYP *)0;i++) {
     IPTYP *ipl;

     ipl=(IPTYP *)calloc(1,sizeof(IPTYP));
     ipl->ip=iplst[i]->ip;
     ipl->mask=iplst[i]->mask;
     dupip=(IPTYP **)rou_addveclst((void **)dupip,(void *)ipl);
    }
  }
return dupip;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by an IP	*/
/*	list.					*/
/*						*/
/************************************************/
IPTYP **net_freeiplst(IPTYP **iplst)

{
if (iplst!=(IPTYP **)0) {
  int i;

  for (i=0;iplst[i]!=(IPTYP *)0;i++) {
    (void) free(iplst[i]);
    }
  (void) free(iplst);
  iplst=(IPTYP **)0;
  }
return iplst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if an IP is indeed	*/
/*	in the relayable list provided		*/
/*						*/
/************************************************/
int net_isiniplst(IPTYP **iplst,char *iporgn)

{
int found;

found=false;
if (iplst!=(IPTYP **)0) {
  struct in_addr addrparam;

  if (inet_aton(iporgn,&addrparam)!=0) {
    u_int32_t adr;
    int i;

    adr=ntohl(addrparam.s_addr);
    for (i=0;iplst[i]!=(IPTYP *)0;i++) {
      int mask;

      mask=iplst[i]->mask;
      if (((adr&(~0U<<mask))==(iplst[i]->ip&(~0U<<mask)))==1) {
	found=true;
	break;
	}
      }
    }
  else {
    (void) alert(0,"uninet.c:net_isiniplst, <%s> wrong ip address",iporgn);
    }
  }
return found;
}
